@@ -13,6 +13,7 @@ logging.basicConfig()
 
 from pcs import (
     acl,
+    booth,
     cluster,
     config,
     constraint,
@@ -97,6 +98,7 @@ def main(argv=None):
             "token=", "token_coefficient=", "consensus=", "join=",
             "miss_count_const=", "fail_recv_const=",
             "corosync_conf=", "cluster_conf=",
+            "booth-conf=", "booth-key=",
             "remote", "watchdog=",
             #in pcs status - do not display resorce status on inactive node
             "hide-inactive",
@@ -179,7 +181,11 @@ def main(argv=None):
         "stonith": stonith.stonith_cmd,
         "property": prop.property_cmd,
         "constraint": constraint.constraint_cmd,
-        "acl": acl.acl_cmd,
+        "acl": lambda argv: acl.acl_cmd(
+            utils.get_library_wrapper(),
+            argv,
+            utils.get_modificators()
+        ),
         "status": status.status_cmd,
         "config": config.config_cmd,
         "pcsd": pcsd.pcsd_cmd,
@@ -199,6 +205,11 @@ def main(argv=None):
             args,
             utils.get_modificators()
         ),
+        "booth": lambda argv: booth.booth_cmd(
+            utils.get_library_wrapper(),
+            argv,
+            utils.get_modificators()
+        ),
     }
     if command not in cmd_map:
         usage.main()
