@@ -26,7 +26,7 @@ from pcs import (
     constraint,
     node,
     pcsd,
-    prop,
+    quorum,
     resource,
     settings,
     status,
@@ -36,12 +36,14 @@ from pcs import (
 )
 from pcs.utils import parallel_for_nodes
 from pcs.common import report_codes
-from pcs.cli.common.reports import process_library_reports
+from pcs.cli.common.reports import process_library_reports, build_report_message
 from pcs.lib import (
     pacemaker as lib_pacemaker,
     sbd as lib_sbd,
     reports as lib_reports,
 )
+from pcs.lib.booth import sync as booth_sync
+from pcs.lib.nodes_task import check_corosync_offline_on_nodes
 from pcs.lib.commands.quorum import _add_device_model_net
 from pcs.lib.corosync import (
     config_parser as corosync_conf_utils,
@@ -54,6 +56,7 @@ from pcs.lib.errors import (
 )
 from pcs.lib.external import (
     disable_service,
+    is_systemctl,
     NodeCommunicationException,
     node_communicator_exception_to_report_item,
 )
@@ -143,9 +146,9 @@ def cluster_cmd(argv):
         cluster_report(argv)
     elif (sub_cmd == "quorum"):
         if argv and argv[0] == "unblock":
-            cluster_quorum_unblock(argv[1:])
+            quorum.quorum_unblock_cmd(argv[1:])
         else:
-            usage.cluster(["quorum"])
+            usage.cluster()
             sys.exit(1)
     else:
         usage.cluster()
@@ -877,14 +880,14 @@ def start_cluster(argv):
 
     print("Starting Cluster...")
     service_list = []
-    if utils.is_rhel6():
+    if utils.is_cman_cluster():
 #   Verify that CMAN_QUORUM_TIMEOUT is set, if not, then we set it to 0
         retval, output = getstatusoutput('source /etc/sysconfig/cman ; [ -z "$CMAN_QUORUM_TIMEOUT" ]')
         if retval == 0:
             with open("/etc/sysconfig/cman", "a") as cman_conf_file:
                 cman_conf_file.write("\nCMAN_QUORUM_TIMEOUT=0\n")
 
-        output, retval = utils.run(["service", "cman","start"])
+        output, retval = utils.start_service("cman")
         if retval != 0:
             print(output)
             utils.err("unable to start cman")
@@ -894,7 +897,7 @@ def start_cluster(argv):
             service_list.append("corosync-qdevice")
     service_list.append("pacemaker")
     for service in service_list:
-        output, retval = utils.run(["service", service, "start"])
+        output, retval = utils.start_service(service)
         if retval != 0:
             print(output)
             utils.err("unable to start {0}".format(service))
@@ -915,9 +918,11 @@ def start_cluster_all():
         wait_for_nodes_started(all_nodes, wait_timeout)
 
 def start_cluster_nodes(nodes):
-    error_list = parallel_for_nodes(utils.startCluster, nodes, quiet=True)
-    if error_list:
-        utils.err("unable to start all nodes\n" + "\n".join(error_list))
+    node_errors = parallel_for_nodes(utils.startCluster, nodes, quiet=True)
+    if node_errors:
+        utils.err(
+            "unable to start all nodes\n" + "\n".join(node_errors.values())
+        )
 
 def is_node_fully_started(node_status):
     return (
@@ -939,7 +944,7 @@ def wait_for_local_node_started(stop_at, interval):
             time.sleep(interval)
     except LibraryError as e:
         return 1, "Unable to get node status: {0}".format(
-            "\n".join([item.message for item in e.args])
+            "\n".join([build_report_message(item) for item in e.args])
         )
 
 def wait_for_remote_node_started(node, stop_at, interval):
@@ -973,10 +978,10 @@ def wait_for_nodes_started(node_list, timeout=None):
         else:
             print(output)
     else:
-        error_list = parallel_for_nodes(
+        node_errors = parallel_for_nodes(
             wait_for_remote_node_started, node_list, stop_at, interval
         )
-        if error_list:
+        if node_errors:
             utils.err("unable to verify all nodes have started")
 
 def stop_cluster_all():
@@ -1030,13 +1035,32 @@ def stop_cluster_nodes(nodes):
                 + "\n".join(error_list)
             )
 
-    error_list = parallel_for_nodes(utils.stopPacemaker, nodes, quiet=True)
-    if error_list:
-        utils.err("unable to stop all nodes\n" + "\n".join(error_list))
+    was_error = False
+    node_errors = parallel_for_nodes(utils.stopPacemaker, nodes, quiet=True)
+    accessible_nodes = [
+        node for node in nodes if node not in node_errors.keys()
+    ]
+    if node_errors:
+        utils.err(
+            "unable to stop all nodes\n" + "\n".join(node_errors.values()),
+            exit_after_error=not accessible_nodes
+        )
+        was_error = True
 
-    error_list = parallel_for_nodes(utils.stopCorosync, nodes, quiet=True)
-    if error_list:
-        utils.err("unable to stop all nodes\n" + "\n".join(error_list))
+    for node in node_errors.keys():
+        print("{0}: Not stopping cluster - node is unreachable".format(node))
+
+    node_errors = parallel_for_nodes(
+        utils.stopCorosync,
+        accessible_nodes,
+        quiet=True
+    )
+    if node_errors:
+        utils.err(
+            "unable to stop all nodes\n" + "\n".join(node_errors.values())
+        )
+    if was_error:
+        utils.err("unable to stop all nodes")
 
 def enable_cluster(argv):
     if len(argv) > 0:
@@ -1074,16 +1098,26 @@ def disable_cluster_nodes(nodes):
     if len(error_list) > 0:
         utils.err("unable to disable all nodes\n" + "\n".join(error_list))
 
-def destroy_cluster(argv):
+def destroy_cluster(argv, keep_going=False):
     if len(argv) > 0:
         # stop pacemaker and resources while cluster is still quorate
         nodes = argv
-        error_list = parallel_for_nodes(utils.stopPacemaker, nodes, quiet=True)
+        node_errors = parallel_for_nodes(utils.stopPacemaker, nodes, quiet=True)
         # proceed with destroy regardless of errors
         # destroy will stop any remaining cluster daemons
-        error_list = parallel_for_nodes(utils.destroyCluster, nodes, quiet=True)
-        if error_list:
-            utils.err("unable to destroy cluster\n" + "\n".join(error_list))
+        node_errors = parallel_for_nodes(utils.destroyCluster, nodes, quiet=True)
+        if node_errors:
+            if keep_going:
+                print(
+                    "Warning: unable to destroy cluster\n"
+                    +
+                    "\n".join(node_errors.values())
+                )
+            else:
+                utils.err(
+                    "unable to destroy cluster\n"
+                    + "\n".join(node_errors.values())
+                )
 
 def stop_cluster(argv):
     if len(argv) > 0:
@@ -1132,7 +1166,18 @@ def stop_cluster(argv):
 
 def stop_cluster_pacemaker():
     print("Stopping Cluster (pacemaker)...")
-    output, retval = utils.run(["service", "pacemaker","stop"])
+    if not is_systemctl():
+        command = ["service", "pacemaker", "stop"]
+        # If --skip-cman is not specified, pacemaker init script will stop cman
+        # and corosync as well. That way some of the nodes may stop cman before
+        # others stop pacemaker, which leads to quorum loss. We need to keep
+        # quorum until all pacemaker resources are stopped as some of them may
+        # need quorum to be able to stop.
+        if utils.is_cman_cluster():
+            command.append("--skip-cman")
+    else:
+        command = ["systemctl", "stop", "pacemaker"]
+    output, retval = utils.run(command)
     if retval != 0:
         print(output)
         utils.err("unable to stop pacemaker")
@@ -1140,7 +1185,7 @@ def stop_cluster_pacemaker():
 def stop_cluster_corosync():
     if utils.is_rhel6():
         print("Stopping Cluster (cman)...")
-        output, retval = utils.run(["service", "cman","stop"])
+        output, retval = utils.stop_service("cman")
         if retval != 0:
             print(output)
             utils.err("unable to stop cman")
@@ -1151,13 +1196,24 @@ def stop_cluster_corosync():
             service_list.append("corosync-qdevice")
         service_list.append("corosync")
         for service in service_list:
-            output, retval = utils.run(["service", service, "stop"])
+            output, retval = utils.stop_service(service)
             if retval != 0:
                 print(output)
                 utils.err("unable to stop {0}".format(service))
 
 def kill_cluster(argv):
-    daemons = ["crmd", "pengine", "attrd", "lrmd", "stonithd", "cib", "pacemakerd", "corosync-qdevice", "corosync"]
+    daemons = [
+        "crmd",
+        "pengine",
+        "attrd",
+        "lrmd",
+        "stonithd",
+        "cib",
+        "pacemakerd",
+        "pacemaker_remoted",
+        "corosync-qdevice",
+        "corosync",
+    ]
     dummy_output, dummy_retval = utils.run(["killall", "-9"] + daemons)
 #    if dummy_retval != 0:
 #        print "Error: unable to execute killall -9"
@@ -1309,6 +1365,37 @@ def get_cib(argv):
         except IOError as e:
             utils.err("Unable to write to file '%s', %s" % (filename, e.strerror))
 
+
+def _ensure_cluster_is_offline_if_atb_should_be_enabled(
+    lib_env, node_num_modifier, skip_offline_nodes=False
+):
+    """
+    Check if cluster is offline if auto tie breaker should be enabled.
+    Raises LibraryError if ATB needs to be enabled cluster is not offline.
+
+    lib_env -- LibraryEnvironment
+    node_num_modifier -- number which wil be added to the number of nodes in
+        cluster when determining whenever ATB is needed.
+    skip_offline_nodes -- if True offline nodes will be skipped
+    """
+    if not lib_env.is_cman_cluster:
+        corosync_conf = lib_env.get_corosync_conf()
+        if lib_sbd.atb_has_to_be_enabled(
+            lib_env.cmd_runner(), corosync_conf, node_num_modifier
+        ):
+            print(
+                "Warning: auto_tie_breaker quorum option will be enabled to "
+                "make SBD fencing effecive after this change. Cluster has to "
+                "be offline to be able to make this change."
+            )
+            check_corosync_offline_on_nodes(
+                lib_env.node_communicator(),
+                lib_env.report_processor,
+                corosync_conf.get_nodes(),
+                skip_offline_nodes
+            )
+
+
 def cluster_node(argv):
     if len(argv) != 2:
         usage.cluster()
@@ -1324,19 +1411,28 @@ def cluster_node(argv):
 
     node = argv[1]
     node0, node1 = utils.parse_multiring_node(node)
-
     if not node0:
         utils.err("missing ring 0 address of the node")
-    status,output = utils.checkAuthorization(node0)
-    if status == 2:
-        utils.err("pcsd is not running on %s" % node0)
-    elif status == 3:
-        utils.err(
-            "%s is not yet authenticated (try pcs cluster auth %s)"
-            % (node0, node0)
-        )
-    elif status != 0:
-        utils.err(output)
+
+    # allow to continue if removing a node with --force
+    if add_node or "--force" not in utils.pcs_options:
+        status, output = utils.checkAuthorization(node0)
+        if status != 0:
+            if status == 2:
+                msg = "pcsd is not running on {0}".format(node0)
+            elif status == 3:
+                msg = (
+                    "{node} is not yet authenticated "
+                    + " (try pcs cluster auth {node})"
+                ).format(node=node0)
+            else:
+                msg = output
+            if not add_node:
+                msg += ", use --force to override"
+            utils.err(msg)
+
+    lib_env = utils.get_lib_env()
+    modifiers = utils.get_modificators()
 
     if add_node == True:
         wait = False
@@ -1355,16 +1451,36 @@ def cluster_node(argv):
                 "cluster is not configured for RRP, "
                 "you must not specify ring 1 address for the node"
             )
-        corosync_conf = None
         (canAdd, error) =  utils.canAddNodeToCluster(node0)
         if not canAdd:
             utils.err("Unable to add '%s' to cluster: %s" % (node0, error))
 
-        lib_env = utils.get_lib_env()
         report_processor = lib_env.report_processor
         node_communicator = lib_env.node_communicator()
         node_addr = NodeAddresses(node0, node1)
+
+        # First set up everything else than corosync. Once the new node is
+        # present in corosync.conf / cluster.conf, it's considered part of a
+        # cluster and the node add command cannot be run again. So we need to
+        # minimize the amout of actions (and therefore possible failures) after
+        # adding the node to corosync.
         try:
+            # qdevice setup
+            if not utils.is_rhel6():
+                conf_facade = corosync_conf_facade.from_string(
+                    utils.getCorosyncConf()
+                )
+                qdevice_model, qdevice_model_options, _ = conf_facade.get_quorum_device_settings()
+                if qdevice_model == "net":
+                    _add_device_model_net(
+                        lib_env,
+                        qdevice_model_options["host"],
+                        conf_facade.get_cluster_name(),
+                        [node_addr],
+                        skip_offline_nodes=False
+                    )
+
+            # sbd setup
             if lib_sbd.is_sbd_enabled(utils.cmd_runner()):
                 if "--watchdog" not in utils.pcs_options:
                     watchdog = settings.sbd_watchdog_default
@@ -1374,6 +1490,10 @@ def cluster_node(argv):
                 else:
                     watchdog = utils.pcs_options["--watchdog"][0]
 
+                _ensure_cluster_is_offline_if_atb_should_be_enabled(
+                    lib_env, 1, modifiers["skip_offline_nodes"]
+                )
+
                 report_processor.process(lib_reports.sbd_check_started())
                 lib_sbd.check_sbd_on_node(
                     report_processor, node_communicator, node_addr, watchdog
@@ -1381,12 +1501,15 @@ def cluster_node(argv):
                 sbd_cfg = environment_file_to_dict(
                     lib_sbd.get_local_sbd_config()
                 )
-                sbd_cfg["SBD_WATCHDOG_DEV"] = watchdog
                 report_processor.process(
                     lib_reports.sbd_config_distribution_started()
                 )
                 lib_sbd.set_sbd_config_on_node(
-                    report_processor, node_communicator, node_addr, sbd_cfg
+                    report_processor,
+                    node_communicator,
+                    node_addr,
+                    sbd_cfg,
+                    watchdog
                 )
                 report_processor.process(lib_reports.sbd_enabling_started())
                 lib_sbd.enable_sbd_service_on_node(
@@ -1397,6 +1520,15 @@ def cluster_node(argv):
                 lib_sbd.disable_sbd_service_on_node(
                     report_processor, node_communicator, node_addr
                 )
+
+            # booth setup
+            booth_sync.send_all_config_to_node(
+                node_communicator,
+                report_processor,
+                node_addr,
+                rewrite_existing=modifiers["force"],
+                skip_wrong_config=modifiers["force"]
+            )
         except LibraryError as e:
             process_library_reports(e.args)
         except NodeCommunicationException as e:
@@ -1404,6 +1536,8 @@ def cluster_node(argv):
                 [node_communicator_exception_to_report_item(e)]
             )
 
+        # Now add the new node to corosync.conf / cluster.conf
+        corosync_conf = None
         for my_node in utils.getNodesFromCorosyncConf():
             retval, output = utils.addLocalNode(my_node, node0, node1)
             if retval != 0:
@@ -1439,24 +1573,6 @@ def cluster_node(argv):
                 except:
                     utils.err('Unable to communicate with pcsd')
 
-            # set qdevice-net certificates if needed
-            if not utils.is_rhel6():
-                try:
-                    conf_facade = corosync_conf_facade.from_string(
-                        corosync_conf
-                    )
-                    qdevice_model, qdevice_model_options, _ = conf_facade.get_quorum_device_settings()
-                    if qdevice_model == "net":
-                        _add_device_model_net(
-                            lib_env,
-                            qdevice_model_options["host"],
-                            conf_facade.get_cluster_name(),
-                            [node_addr],
-                            skip_offline_nodes=False
-                        )
-                except LibraryError as e:
-                    process_library_reports(e.args)
-
             print("Setting up corosync...")
             utils.setCorosyncConfig(node0, corosync_conf)
             if "--enable" in utils.pcs_options:
@@ -1515,9 +1631,16 @@ def cluster_node(argv):
                 )
             # else the node seems to be stopped already, we're ok to proceed
 
+        try:
+            _ensure_cluster_is_offline_if_atb_should_be_enabled(
+                lib_env, -1, modifiers["skip_offline_nodes"]
+            )
+        except LibraryError as e:
+            utils.process_library_reports(e.args)
+
         nodesRemoved = False
         c_nodes = utils.getNodesFromCorosyncConf()
-        destroy_cluster([node0])
+        destroy_cluster([node0], keep_going=("--force" in utils.pcs_options))
         for my_node in c_nodes:
             if my_node == node0:
                 continue
@@ -1744,11 +1867,10 @@ def cluster_destroy(argv):
         destroy_cluster(utils.getNodesFromCorosyncConf())
     else:
         print("Shutting down pacemaker/corosync services...")
-        os.system("service pacemaker stop")
-        # returns error if qdevice is not running, it is safe to ignore it
-        # since we want it not to be running
-        os.system("service corosync-qdevice stop")
-        os.system("service corosync stop")
+        for service in ["pacemaker", "corosync-qdevice", "corosync"]:
+            # Returns an error if a service is not running. It is safe to
+            # ignore it since we want it not to be running anyways.
+            utils.stop_service(service)
         print("Killing any remaining services...")
         os.system("killall -q -9 corosync corosync-qdevice aisexec heartbeat pacemakerd ccm stonithd ha_logd lrmd crmd pengine attrd pingd mgmtd cib fenced dlm_controld gfs_controld")
         try:
@@ -1758,7 +1880,7 @@ def cluster_destroy(argv):
             # for now
             pass
         try:
-            disable_service(utils.cmd_runner(), "sbd")
+            disable_service(utils.cmd_runner(), lib_sbd.get_sbd_service_name())
         except:
             # it's not a big deal if sbd disable fails
             pass
@@ -1886,60 +2008,13 @@ def cluster_remote_node(argv):
             nvpair.parentNode.removeChild(nvpair)
         dom = constraint.remove_constraints_containing_node(dom, hostname)
         utils.replace_cib_configuration(dom)
+        if not utils.usefile:
+            output, retval = utils.run([
+                "crm_node", "--force", "--remove", hostname
+            ])
+            if retval != 0:
+                utils.err("unable to remove: {0}".format(output))
     else:
         usage.cluster(["remote-node"])
         sys.exit(1)
 
-def cluster_quorum_unblock(argv):
-    if len(argv) > 0:
-        usage.quorum(["unblock"])
-        sys.exit(1)
-
-    if utils.is_rhel6():
-        utils.err("operation is not supported on CMAN clusters")
-
-    output, retval = utils.run(
-        ["corosync-cmapctl", "-g", "runtime.votequorum.wait_for_all_status"]
-    )
-    if retval != 0:
-        utils.err("unable to check quorum status")
-    if output.split("=")[-1].strip() != "1":
-        utils.err("cluster is not waiting for nodes to establish quorum")
-
-    unjoined_nodes = (
-        set(utils.getNodesFromCorosyncConf())
-        -
-        set(utils.getCorosyncActiveNodes())
-    )
-    if not unjoined_nodes:
-        utils.err("no unjoined nodes found")
-    if "--force" not in utils.pcs_options:
-        answer = utils.get_terminal_input(
-            (
-                "WARNING: If node(s) {nodes} are not powered off or they do"
-                + " have access to shared resources, data corruption and/or"
-                + " cluster failure may occur. Are you sure you want to"
-                + " continue? [y/N] "
-            ).format(nodes=", ".join(unjoined_nodes))
-        )
-        if answer.lower() not in ["y", "yes"]:
-            print("Canceled")
-            return
-    for node in unjoined_nodes:
-        stonith.stonith_confirm([node], skip_question=True)
-
-    output, retval = utils.run(
-        ["corosync-cmapctl", "-s", "quorum.cancel_wait_for_all", "u8", "1"]
-    )
-    if retval != 0:
-        utils.err("unable to cancel waiting for nodes")
-    print("Quorum unblocked")
-
-    startup_fencing = prop.get_set_properties().get("startup-fencing", "")
-    utils.set_cib_property(
-        "startup-fencing",
-        "false" if startup_fencing.lower() != "false" else "true"
-    )
-    utils.set_cib_property("startup-fencing", startup_fencing)
-    print("Waiting for nodes canceled")
-
