@@ -90,6 +90,7 @@ def constraint_cmd(argv):
             command_map = {
                 "set": ticket_command.create_with_set,
                 "add": ticket_command.add,
+                "remove": ticket_command.remove,
                 "show": ticket_command.show,
             }
             sub_command = argv[0] if argv else "show"
@@ -593,7 +594,7 @@ def location_show(argv):
             print("  Node: " + node)
 
             nodehash_label = (
-                (nodehashon, "    Allowed to run:")
+                (nodehashon, "    Allowed to run:"),
                 (nodehashoff, "    Not allowed to run:")
             )
             for nodehash, label in nodehash_label:
