@@ -11,6 +11,7 @@ import tempfile
 from lxml import etree
 
 from pcs import settings
+from pcs.common.tools import join_multilines
 from pcs.lib import reports
 from pcs.lib.errors import LibraryError
 from pcs.lib.pacemaker_values import validate_id
@@ -21,7 +22,17 @@ def does_id_exist(tree, check_id):
     tree cib etree node
     check_id id to check
     """
-    return tree.find('.//*[@id="{0}"]'.format(check_id)) is not None
+    # ElementTree has getroot, Elemet has getroottree
+    root = tree.getroot() if hasattr(tree, "getroot") else tree.getroottree()
+    # do not search in /cib/status, it may contain references to previously
+    # existing and deleted resources and thus preventing creating them again
+    existing = root.xpath(
+        (
+            '(/cib/*[name()!="status"]|/*[name()!="cib"])' +
+            '//*[name()!="acl_target" and name()!="role" and @id="{0}"]'
+        ).format(check_id)
+    )
+    return len(existing) > 0
 
 def validate_id_does_not_exist(tree, id):
     """
@@ -44,6 +55,12 @@ def find_unique_id(tree, check_id):
         counter += 1
     return temp_id
 
+def create_subelement_id(context_element, suffix):
+    return find_unique_id(
+        context_element,
+        "{0}-{1}".format(context_element.get("id"), suffix)
+    )
+
 def check_new_id_applicable(tree, description, id):
     validate_id(id, description)
     validate_id_does_not_exist(tree, id)
@@ -91,6 +108,13 @@ def get_constraints(tree):
     """
     return _get_mandatory_section(tree, "configuration/constraints")
 
+def get_resources(tree):
+    """
+    Return 'resources' element from tree
+    tree cib etree node
+    """
+    return _get_mandatory_section(tree, "configuration/resources")
+
 def find_parent(element, tag_names):
     candidate = element
     while True:
@@ -104,7 +128,7 @@ def export_attributes(element):
 
 def get_sub_element(element, sub_element_tag, new_id=None, new_index=None):
     """
-    Returns sub-element sub_element_tag of element. It will create new
+    Returns the FIRST sub-element sub_element_tag of element. It will create new
     element if such doesn't exist yet. Id of new element will be new_if if
     it's not None. new_index specify where will be new element added, if None
     it will be appended.
@@ -160,29 +184,33 @@ def upgrade_cib(cib, runner):
     cib -- cib etree
     runner -- CommandRunner
     """
-    temp_file = tempfile.NamedTemporaryFile("w+", suffix=".pcs")
-    temp_file.write(etree.tostring(cib).decode())
-    temp_file.flush()
-    output, retval = runner.run(
-        [
-            os.path.join(settings.pacemaker_binaries, "cibadmin"),
-            "--upgrade",
-            "--force"
-        ],
-        env_extend={"CIB_file": temp_file.name}
-    )
-
-    if retval != 0:
-        temp_file.close()
-        LibraryError(reports.cib_upgrade_failed(output))
-
+    temp_file = None
     try:
+        temp_file = tempfile.NamedTemporaryFile("w+", suffix=".pcs")
+        temp_file.write(etree.tostring(cib).decode())
+        temp_file.flush()
+        stdout, stderr, retval = runner.run(
+            [
+                os.path.join(settings.pacemaker_binaries, "cibadmin"),
+                "--upgrade",
+                "--force"
+            ],
+            env_extend={"CIB_file": temp_file.name}
+        )
+
+        if retval != 0:
+            temp_file.close()
+            raise LibraryError(
+                reports.cib_upgrade_failed(join_multilines([stderr, stdout]))
+            )
+
         temp_file.seek(0)
         return etree.fromstring(temp_file.read())
     except (EnvironmentError, etree.XMLSyntaxError, etree.DocumentInvalid) as e:
-        LibraryError(reports.cib_upgrade_failed(str(e)))
+        raise LibraryError(reports.cib_upgrade_failed(str(e)))
     finally:
-        temp_file.close()
+        if temp_file:
+            temp_file.close()
 
 
 def ensure_cib_version(runner, cib, version):
@@ -214,3 +242,17 @@ def ensure_cib_version(runner, cib, version):
     raise LibraryError(reports.unable_to_upgrade_cib_to_required_version(
         current_version, version
     ))
+
+
+def etree_element_attibutes_to_dict(etree_el, required_key_list):
+    """
+    Returns all attributes of etree_el from required_key_list in dictionary,
+    where keys are attributes and values are values of attributes or None if
+    it's not present.
+
+    etree_el -- etree element from which attributes should be extracted
+    required_key_list -- list of strings, attributes names which should be
+        extracted
+    """
+    return dict([(key, etree_el.get(key)) for key in required_key_list])
+
