@@ -8,12 +8,13 @@ from __future__ import (
 import logging
 from lxml import etree
 
-from unittest import TestCase
+from pcs.test.tools.pcs_unittest import TestCase
 
-from pcs.test.tools.pcs_mock import mock
+from pcs.test.tools.pcs_unittest import mock
 from pcs.test.tools.assertions import (
     assert_raise_library_error,
     assert_xml_equal,
+    assert_report_item_list_equal,
 )
 from pcs.test.tools.custom_mock import MockLibraryReportProcessor
 
@@ -278,8 +279,10 @@ class RemoveAlertTest(TestCase):
             <cib validate-with="pacemaker-2.5">
                 <configuration>
                     <alerts>
-                        <alert id="alert" path="path"/>
-                        <alert id="alert-1" path="/path"/>
+                        <alert id="alert1" path="path"/>
+                        <alert id="alert2" path="/path"/>
+                        <alert id="alert3" path="/path"/>
+                        <alert id="alert4" path="/path"/>
                     </alerts>
                 </configuration>
             </cib>
@@ -288,29 +291,80 @@ class RemoveAlertTest(TestCase):
             self.mock_log, self.mock_rep, cib_data=cib
         )
 
-    def test_success(self):
-        cmd_alert.remove_alert(self.mock_env, "alert")
+    def test_one_alert(self):
+        cmd_alert.remove_alert(self.mock_env, ["alert2"])
         assert_xml_equal(
             """
                 <cib validate-with="pacemaker-2.5">
                     <configuration>
                         <alerts>
-                            <alert id="alert-1" path="/path"/>
+                            <alert id="alert1" path="path"/>
+                            <alert id="alert3" path="/path"/>
+                            <alert id="alert4" path="/path"/>
                         </alerts>
                     </configuration>
                 </cib>
             """,
             self.mock_env._get_cib_xml()
         )
+        self.assertEqual([], self.mock_rep.report_item_list)
 
-    def test_not_existing_alert(self):
-        assert_raise_library_error(
-            lambda: cmd_alert.remove_alert(self.mock_env, "unknown"),
+    def test_multiple_alerts(self):
+        cmd_alert.remove_alert(self.mock_env, ["alert1", "alert3", "alert4"])
+        assert_xml_equal(
+            """
+                <cib validate-with="pacemaker-2.5">
+                    <configuration>
+                        <alerts>
+                            <alert id="alert2" path="/path"/>
+                        </alerts>
+                    </configuration>
+                </cib>
+            """,
+            self.mock_env._get_cib_xml()
+        )
+        self.assertEqual([], self.mock_rep.report_item_list)
+
+    def test_no_alert(self):
+        cmd_alert.remove_alert(self.mock_env, [])
+        assert_xml_equal(
+            """
+                <cib validate-with="pacemaker-2.5">
+                    <configuration>
+                        <alerts>
+                            <alert id="alert1" path="path"/>
+                            <alert id="alert2" path="/path"/>
+                            <alert id="alert3" path="/path"/>
+                            <alert id="alert4" path="/path"/>
+                        </alerts>
+                    </configuration>
+                </cib>
+            """,
+            self.mock_env._get_cib_xml()
+        )
+        self.assertEqual([], self.mock_rep.report_item_list)
+
+    def test_failure(self):
+        report_list = [
             (
                 Severities.ERROR,
                 report_codes.CIB_ALERT_NOT_FOUND,
                 {"alert": "unknown"}
+            ),
+            (
+                Severities.ERROR,
+                report_codes.CIB_ALERT_NOT_FOUND,
+                {"alert": "unknown2"}
             )
+        ]
+        assert_raise_library_error(
+            lambda: cmd_alert.remove_alert(
+                self.mock_env, ["unknown", "alert1", "unknown2", "alert2"]
+            ),
+            *report_list
+        )
+        assert_report_item_list_equal(
+            self.mock_rep.report_item_list, report_list
         )
 
 
@@ -361,19 +415,17 @@ class AddRecipientTest(TestCase):
     def test_recipient_already_exists(self):
         assert_raise_library_error(
             lambda: cmd_alert.add_recipient(
-                self.mock_env, "alert", "value1", {}, {}
+                self.mock_env, "alert", "value1", {}, {},
+                recipient_id="alert-recipient"
             ),
             (
                 Severities.ERROR,
-                report_codes.CIB_ALERT_RECIPIENT_ALREADY_EXISTS,
-                {
-                    "recipient": "value1",
-                    "alert": "alert"
-                }
+                report_codes.ID_ALREADY_EXISTS,
+                {"id": "alert-recipient"}
             )
         )
 
-    def test_success(self):
+    def test_without_id(self):
         cmd_alert.add_recipient(
             self.mock_env,
             "alert",
@@ -424,6 +476,58 @@ class AddRecipientTest(TestCase):
             self.mock_env._get_cib_xml()
         )
 
+    def test_with_id(self):
+        cmd_alert.add_recipient(
+            self.mock_env,
+            "alert",
+            "value",
+            {"attr1": "val1"},
+            {
+                "attr2": "val2",
+                "attr1": "val1"
+            },
+            recipient_id="my-recipient"
+        )
+        assert_xml_equal(
+            """
+<cib validate-with="pacemaker-2.5">
+    <configuration>
+        <alerts>
+            <alert id="alert" path="path">
+                <recipient id="alert-recipient" value="value1"/>
+                <recipient id="my-recipient" value="value">
+                    <meta_attributes
+                        id="my-recipient-meta_attributes"
+                    >
+                        <nvpair
+                            id="my-recipient-meta_attributes-attr1"
+                            name="attr1"
+                            value="val1"
+                        />
+                        <nvpair
+                            id="my-recipient-meta_attributes-attr2"
+                            name="attr2"
+                            value="val2"
+                        />
+                    </meta_attributes>
+                    <instance_attributes
+                        id="my-recipient-instance_attributes"
+                    >
+                        <nvpair
+                            id="my-recipient-instance_attributes-attr1"
+                            name="attr1"
+                            value="val1"
+                        />
+                    </instance_attributes>
+                </recipient>
+            </alert>
+        </alerts>
+    </configuration>
+</cib>
+            """,
+            self.mock_env._get_cib_xml()
+        )
+
 
 class UpdateRecipientTest(TestCase):
     def setUp(self):
@@ -470,29 +574,29 @@ class UpdateRecipientTest(TestCase):
             self.mock_log, self.mock_rep, cib_data=cib
         )
 
-    def test_alert_not_found(self):
+    def test_empty_value(self):
         assert_raise_library_error(
             lambda: cmd_alert.update_recipient(
-                self.mock_env, "unknown", "recipient", {}, {}
+                self.mock_env, "alert-recipient-1", {}, {}, recipient_value=""
             ),
             (
                 Severities.ERROR,
-                report_codes.CIB_ALERT_NOT_FOUND,
-                {"alert": "unknown"}
+                report_codes.CIB_ALERT_RECIPIENT_VALUE_INVALID,
+                {"recipient": ""}
             )
         )
 
     def test_recipient_not_found(self):
         assert_raise_library_error(
             lambda: cmd_alert.update_recipient(
-                self.mock_env, "alert", "recipient", {}, {}
+                self.mock_env, "recipient", {}, {}
             ),
             (
                 Severities.ERROR,
-                report_codes.CIB_ALERT_RECIPIENT_NOT_FOUND,
+                report_codes.ID_NOT_FOUND,
                 {
-                    "recipient": "recipient",
-                    "alert": "alert"
+                    "id": "recipient",
+                    "id_description": "Recipient"
                 }
             )
         )
@@ -500,14 +604,14 @@ class UpdateRecipientTest(TestCase):
     def test_update_all(self):
         cmd_alert.update_recipient(
             self.mock_env,
-            "alert",
-            "value",
+            "alert-recipient-1",
             {"attr1": "value"},
             {
                 "attr1": "",
                 "attr3": "new_val"
             },
-            "desc"
+            recipient_value="new_val",
+            description="desc"
         )
         assert_xml_equal(
             """
@@ -518,7 +622,7 @@ class UpdateRecipientTest(TestCase):
                 <recipient id="alert-recipient" value="value1"/>
                 <recipient
                     id="alert-recipient-1"
-                    value="value"
+                    value="new_val"
                     description="desc"
                 >
                     <meta_attributes
@@ -564,8 +668,12 @@ class RemoveRecipientTest(TestCase):
                 <configuration>
                     <alerts>
                         <alert id="alert" path="path">
-                            <recipient id="alert-recipient" value="value1"/>
-                            <recipient id="alert-recipient-1" value="value"/>
+                            <recipient id="alert-recipient1" value="value1"/>
+                            <recipient id="alert-recipient2" value="value2"/>
+                        </alert>
+                        <alert id="alert2" path="path">
+                            <recipient id="alert2-recipient3" value="value3"/>
+                            <recipient id="alert2-recipient4" value="value4"/>
                         </alert>
                     </alerts>
                 </configuration>
@@ -575,42 +683,43 @@ class RemoveRecipientTest(TestCase):
             self.mock_log, self.mock_rep, cib_data=cib
         )
 
-    def test_alert_not_found(self):
-        assert_raise_library_error(
-            lambda: cmd_alert.remove_recipient(
-                self.mock_env, "unknown", "recipient"
-            ),
-            (
-                Severities.ERROR,
-                report_codes.CIB_ALERT_NOT_FOUND,
-                {"alert": "unknown"}
-            )
-        )
-
     def test_recipient_not_found(self):
-        assert_raise_library_error(
-            lambda: cmd_alert.remove_recipient(
-                self.mock_env, "alert", "recipient"
+        report_list = [
+            (
+                Severities.ERROR,
+                report_codes.ID_NOT_FOUND,
+                {"id": "recipient"}
             ),
             (
                 Severities.ERROR,
-                report_codes.CIB_ALERT_RECIPIENT_NOT_FOUND,
-                {
-                    "recipient": "recipient",
-                    "alert": "alert"
-                }
+                report_codes.ID_NOT_FOUND,
+                {"id": "alert2-recipient1"}
             )
+        ]
+        assert_raise_library_error(
+            lambda: cmd_alert.remove_recipient(
+                self.mock_env,
+                ["recipient", "alert-recipient1", "alert2-recipient1"]
+            ),
+            *report_list
+        )
+        assert_report_item_list_equal(
+            self.mock_rep.report_item_list, report_list
         )
 
-    def test_success(self):
-        cmd_alert.remove_recipient(self.mock_env, "alert", "value1")
+    def test_one_recipient(self):
+        cmd_alert.remove_recipient(self.mock_env, ["alert-recipient1"])
         assert_xml_equal(
             """
             <cib validate-with="pacemaker-2.5">
                 <configuration>
                     <alerts>
                         <alert id="alert" path="path">
-                            <recipient id="alert-recipient-1" value="value"/>
+                            <recipient id="alert-recipient2" value="value2"/>
+                        </alert>
+                        <alert id="alert2" path="path">
+                            <recipient id="alert2-recipient3" value="value3"/>
+                            <recipient id="alert2-recipient4" value="value4"/>
                         </alert>
                     </alerts>
                 </configuration>
@@ -618,6 +727,52 @@ class RemoveRecipientTest(TestCase):
             """,
             self.mock_env._get_cib_xml()
         )
+        self.assertEqual([], self.mock_rep.report_item_list)
+
+    def test_multiple_recipients(self):
+        cmd_alert.remove_recipient(
+            self.mock_env,
+            ["alert-recipient1", "alert-recipient2", "alert2-recipient4"]
+        )
+        assert_xml_equal(
+            """
+            <cib validate-with="pacemaker-2.5">
+                <configuration>
+                    <alerts>
+                        <alert id="alert" path="path"/>
+                        <alert id="alert2" path="path">
+                            <recipient id="alert2-recipient3" value="value3"/>
+                        </alert>
+                    </alerts>
+                </configuration>
+            </cib>
+            """,
+            self.mock_env._get_cib_xml()
+        )
+        self.assertEqual([], self.mock_rep.report_item_list)
+
+    def test_no_recipient(self):
+        cmd_alert.remove_recipient(self.mock_env, [])
+        assert_xml_equal(
+            """
+            <cib validate-with="pacemaker-2.5">
+                <configuration>
+                    <alerts>
+                        <alert id="alert" path="path">
+                            <recipient id="alert-recipient1" value="value1"/>
+                            <recipient id="alert-recipient2" value="value2"/>
+                        </alert>
+                        <alert id="alert2" path="path">
+                            <recipient id="alert2-recipient3" value="value3"/>
+                            <recipient id="alert2-recipient4" value="value4"/>
+                        </alert>
+                    </alerts>
+                </configuration>
+            </cib>
+            """,
+            self.mock_env._get_cib_xml()
+        )
+        self.assertEqual([], self.mock_rep.report_item_list)
 
 
 @mock.patch("pcs.lib.cib.alert.get_all_alerts")
