@@ -47,21 +47,28 @@ except ImportError:
         URLError as urllib_URLError
     )
 
-from pcs.lib import reports
-from pcs.lib.errors import LibraryError, ReportItemSeverity
+from pcs import settings
 from pcs.common import report_codes
 from pcs.common.tools import (
+    join_multilines,
     simple_cache,
     run_parallel as tools_run_parallel,
 )
-from pcs import settings
+from pcs.lib import reports
+from pcs.lib.errors import LibraryError, ReportItemSeverity
 
 
+
+_chkconfig = settings.chkconfig_binary
+_service = settings.service_binary
+_systemctl = settings.systemctl_binary
+
 class ManageServiceError(Exception):
     #pylint: disable=super-init-not-called
-    def __init__(self, service, message=None):
+    def __init__(self, service, message=None, instance=None):
         self.service = service
         self.message = message
+        self.instance = instance
 
 class DisableServiceError(ManageServiceError):
     pass
@@ -79,10 +86,6 @@ class KillServicesError(ManageServiceError):
     pass
 
 
-def is_path_runnable(path):
-    return os.path.isfile(path) and os.access(path, os.X_OK)
-
-
 def is_dir_nonempty(path):
     if not os.path.exists(path):
         return False
@@ -91,6 +94,22 @@ def is_dir_nonempty(path):
     return len(os.listdir(path)) > 0
 
 
+def _get_service_name(service, instance=None):
+    return "{0}{1}.service".format(
+        service, "" if instance is None else "@{0}".format(instance)
+    )
+
+def ensure_is_systemd():
+    """
+    Ensure if current system is systemd system. Raises Library error if not.
+    """
+    if not is_systemctl():
+        raise LibraryError(
+            reports.unsupported_operation_on_non_systemd_systems()
+        )
+
+
+
 @simple_cache
 def is_systemctl():
     """
@@ -101,6 +120,7 @@ def is_systemctl():
         '/usr/bin/systemctl',
         '/bin/systemctl',
         '/var/run/systemd/system',
+        '/run/systemd/system',
     ]
     for path in systemctl_paths:
         if os.path.exists(path):
@@ -108,74 +128,98 @@ def is_systemctl():
     return False
 
 
-def disable_service(runner, service):
+def disable_service(runner, service, instance=None):
     """
     Disable specified service in local system.
     Raise DisableServiceError or LibraryError on failure.
 
     runner -- CommandRunner
     service -- name of service
+    instance -- instance name, it ha no effect on not systemd systems.
+        If None no instance name will be used.
     """
+    if not is_service_installed(runner, service):
+        return
     if is_systemctl():
-        output, retval = runner.run([
-            "systemctl", "disable", service + ".service"
+        stdout, stderr, retval = runner.run([
+            _systemctl, "disable", _get_service_name(service, instance)
         ])
     else:
-        if not is_service_installed(runner, service):
-            return
-        output, retval = runner.run(["chkconfig", service, "off"])
+        stdout, stderr, retval = runner.run([_chkconfig, service, "off"])
     if retval != 0:
-        raise DisableServiceError(service, output.rstrip())
+        raise DisableServiceError(
+            service,
+            join_multilines([stderr, stdout]),
+            instance
+        )
 
 
-def enable_service(runner, service):
+def enable_service(runner, service, instance=None):
     """
     Enable specified service in local system.
     Raise EnableServiceError or LibraryError on failure.
 
     runner -- CommandRunner
     service -- name of service
+    instance -- instance name, it ha no effect on not systemd systems.
+        If None no instance name will be used.
     """
     if is_systemctl():
-        output, retval = runner.run([
-            "systemctl", "enable", service + ".service"
+        stdout, stderr, retval = runner.run([
+            _systemctl, "enable", _get_service_name(service, instance)
         ])
     else:
-        output, retval = runner.run(["chkconfig", service, "on"])
+        stdout, stderr, retval = runner.run([_chkconfig, service, "on"])
     if retval != 0:
-        raise EnableServiceError(service, output.rstrip())
+        raise EnableServiceError(
+            service,
+            join_multilines([stderr, stdout]),
+            instance
+        )
 
 
-def start_service(runner, service):
+def start_service(runner, service, instance=None):
     """
     Start specified service in local system
     CommandRunner runner
     string service service name
+    string instance instance name, it ha no effect on not systemd systems.
+        If None no instance name will be used.
     """
     if is_systemctl():
-        output, retval = runner.run([
-            "systemctl", "start", "{0}.service".format(service)
+        stdout, stderr, retval = runner.run([
+            _systemctl, "start", _get_service_name(service, instance)
         ])
     else:
-        output, retval = runner.run(["service", service, "start"])
+        stdout, stderr, retval = runner.run([_service, service, "start"])
     if retval != 0:
-        raise StartServiceError(service, output.rstrip())
+        raise StartServiceError(
+            service,
+            join_multilines([stderr, stdout]),
+            instance
+        )
 
 
-def stop_service(runner, service):
+def stop_service(runner, service, instance=None):
     """
     Stop specified service in local system
     CommandRunner runner
     string service service name
+    string instance instance name, it ha no effect on not systemd systems.
+        If None no instance name will be used.
     """
     if is_systemctl():
-        output, retval = runner.run([
-            "systemctl", "stop", "{0}.service".format(service)
+        stdout, stderr, retval = runner.run([
+            _systemctl, "stop", _get_service_name(service, instance)
         ])
     else:
-        output, retval = runner.run(["service", service, "stop"])
+        stdout, stderr, retval = runner.run([_service, service, "stop"])
     if retval != 0:
-        raise StopServiceError(service, output.rstrip())
+        raise StopServiceError(
+            service,
+            join_multilines([stderr, stdout]),
+            instance
+        )
 
 
 def kill_services(runner, services):
@@ -185,18 +229,19 @@ def kill_services(runner, services):
     iterable services service names
     """
     # make killall not report that a process is not running
-    output, retval = runner.run(
+    stdout, stderr, retval = runner.run(
         ["killall", "--quiet", "--signal", "9", "--"] + list(services)
     )
     # If a process isn't running, killall will still return 1 even with --quiet.
     # We don't consider that an error, so we check for output string as well.
     # If it's empty, no actuall error happened.
     if retval != 0:
-        if output.strip():
-            raise KillServicesError(list(services), output.rstrip())
+        message = join_multilines([stderr, stdout])
+        if message:
+            raise KillServicesError(list(services), message)
 
 
-def is_service_enabled(runner, service):
+def is_service_enabled(runner, service, instance=None):
     """
     Check if specified service is enabled in local system.
 
@@ -204,16 +249,16 @@ def is_service_enabled(runner, service):
     service -- name of service
     """
     if is_systemctl():
-        _, retval = runner.run(
-            ["systemctl", "is-enabled", service + ".service"]
+        dummy_stdout, dummy_stderr, retval = runner.run(
+            [_systemctl, "is-enabled", _get_service_name(service, instance)]
         )
     else:
-        _, retval = runner.run(["chkconfig", service])
+        dummy_stdout, dummy_stderr, retval = runner.run([_chkconfig, service])
 
     return retval == 0
 
 
-def is_service_running(runner, service):
+def is_service_running(runner, service, instance=None):
     """
     Check if specified service is currently running on local system.
 
@@ -221,9 +266,15 @@ def is_service_running(runner, service):
     service -- name of service
     """
     if is_systemctl():
-        _, retval = runner.run(["systemctl", "is-active", service + ".service"])
+        dummy_stdout, dummy_stderr, retval = runner.run([
+            _systemctl,
+            "is-active",
+            _get_service_name(service, instance)
+        ])
     else:
-        _, retval = runner.run(["service", service, "status"])
+        dummy_stdout, dummy_stderr, retval = runner.run(
+            [_service, service, "status"]
+        )
 
     return retval == 0
 
@@ -250,12 +301,12 @@ def get_non_systemd_services(runner):
     if is_systemctl():
         return []
 
-    output, return_code = runner.run(["chkconfig"], ignore_stderr=True)
+    stdout, dummy_stderr, return_code = runner.run([_chkconfig])
     if return_code != 0:
         return []
 
     service_list = []
-    for service in output.splitlines():
+    for service in stdout.splitlines():
         service = service.split(" ", 1)[0]
         if service:
             service_list.append(service)
@@ -271,12 +322,14 @@ def get_systemd_services(runner):
     if not is_systemctl():
         return []
 
-    output, return_code = runner.run(["systemctl", "list-unit-files", "--full"])
+    stdout, dummy_stderr, return_code = runner.run([
+        _systemctl, "list-unit-files", "--full"
+    ])
     if return_code != 0:
         return []
 
     service_list = []
-    for service in output.splitlines():
+    for service in stdout.splitlines():
         match = re.search(r'^([\S]*)\.service', service)
         if match:
             service_list.append(match.group(1))
@@ -293,13 +346,13 @@ def is_cman_cluster(runner):
     # - corosync1 runs with cman on rhel6
     # - corosync1 can be used without cman, but we don't support it anyways
     # - corosync2 is the default result if errors occur
-    output, retval = runner.run([
+    stdout, dummy_stderr, retval = runner.run([
         os.path.join(settings.corosync_binaries, "corosync"),
         "-v"
     ])
     if retval != 0:
         return False
-    match = re.search(r"version\D+(\d+)", output)
+    match = re.search(r"version\D+(\d+)", stdout)
     return match is not None and match.group(1) == "1"
 
 
@@ -307,15 +360,25 @@ class CommandRunner(object):
     def __init__(self, logger, reporter, env_vars=None):
         self._logger = logger
         self._reporter = reporter
+        # Reset environment variables by empty dict is desired here.  We need
+        # to get rid of defaults - we do not know the context and environment
+        # where the library runs.  We also get rid of PATH settings, so all
+        # executables must be specified with full path unless the PATH variable
+        # is set from outside.
         self._env_vars = env_vars if env_vars else dict()
         self._python2 = sys.version[0] == "2"
 
     def run(
-        self, args, ignore_stderr=False, stdin_string=None, env_extend=None,
-        binary_output=False
+        self, args, stdin_string=None, env_extend=None, binary_output=False
     ):
-        env_vars = dict(env_extend) if env_extend else dict()
-        env_vars.update(self._env_vars)
+        # Allow overriding default settings. If a piece of code really wants to
+        # set own PATH or CIB_file, we must allow it. I.e. it wants to run
+        # a pacemaker tool on a CIB in a file but cannot afford the risk of
+        # changing the CIB in the file specified by the user.
+        env_vars = self._env_vars
+        env_vars.update(
+            dict(env_extend) if env_extend else dict()
+        )
 
         log_args = " ".join([shell_quote(x) for x in args])
         msg = "Running: {args}"
@@ -332,9 +395,7 @@ class CommandRunner(object):
                 # Some commands react differently if they get anything via stdin
                 stdin=(subprocess.PIPE if stdin_string is not None else None),
                 stdout=subprocess.PIPE,
-                stderr=(
-                    subprocess.PIPE if ignore_stderr else subprocess.STDOUT
-                ),
+                stderr=subprocess.PIPE,
                 preexec_fn=(
                     lambda: signal.signal(signal.SIGPIPE, signal.SIG_DFL)
                 ),
@@ -344,7 +405,7 @@ class CommandRunner(object):
                 # decodes newlines and in python3 also converts bytes to str
                 universal_newlines=(not self._python2 and not binary_output)
             )
-            output, dummy_stderror = process.communicate(stdin_string)
+            out_std, out_err = process.communicate(stdin_string)
             retval = process.returncode
         except OSError as e:
             raise LibraryError(
@@ -354,13 +415,19 @@ class CommandRunner(object):
         self._logger.debug(
             (
                 "Finished running: {args}\nReturn value: {retval}"
-                + "\n--Debug Output Start--\n{output}\n--Debug Output End--"
-            ).format(args=log_args, retval=retval, output=output)
+                + "\n--Debug Stdout Start--\n{out_std}\n--Debug Stdout End--"
+                + "\n--Debug Stderr Start--\n{out_err}\n--Debug Stderr End--"
+            ).format(
+                args=log_args,
+                retval=retval,
+                out_std=out_std,
+                out_err=out_err
+            )
         )
-        self._reporter.process(
-            reports.run_external_process_finished(log_args, retval, output)
-        )
-        return output, retval
+        self._reporter.process(reports.run_external_process_finished(
+            log_args, retval, out_std, out_err
+        ))
+        return out_std, out_err, retval
 
 
 class NodeCommunicationException(Exception):
