@@ -1,16 +1,17 @@
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
+from __future__ import (
+    absolute_import,
+    division,
+    print_function,
+    unicode_literals,
+)
 
 import sys
 import os
 import errno
-import json
 
-import usage
-import utils
-import settings
+from pcs import usage
+from pcs import utils
+from pcs import settings
 
 
 def pcsd_cmd(argv):
@@ -52,7 +53,7 @@ def pcsd_certkey(argv):
             utils.err(err, False)
         sys.exit(1)
 
-    if not "--force" in utils.pcs_options and (os.path.exists(settings.pcsd_cert_location) or os.path.exists(settings.pcsd_key_location)):
+    if "--force" not in utils.pcs_options and (os.path.exists(settings.pcsd_cert_location) or os.path.exists(settings.pcsd_key_location)):
         utils.err("certificate and/or key already exists, your must use --force to overwrite")
 
     try:
