@@ -5,12 +5,12 @@ from __future__ import (
     unicode_literals,
 )
 
-from unittest import TestCase
+from pcs.test.tools.pcs_unittest import TestCase
 
 import base64
 import os.path
 
-from pcs.test.tools.pcs_mock import mock
+from pcs.test.tools.pcs_unittest import mock
 from pcs.test.tools.assertions import assert_raise_library_error
 from pcs.test.tools.misc import get_test_resource
 
@@ -49,7 +49,7 @@ class QdeviceSetupTest(TestCase):
 
     def test_success(self, mock_is_dir_nonempty):
         mock_is_dir_nonempty.return_value = False
-        self.mock_runner.run.return_value = ("initialized", 0)
+        self.mock_runner.run.return_value = ("initialized", "", 0)
 
         lib.qdevice_setup(self.mock_runner)
 
@@ -73,7 +73,7 @@ class QdeviceSetupTest(TestCase):
 
     def test_init_tool_fail(self, mock_is_dir_nonempty):
         mock_is_dir_nonempty.return_value = False
-        self.mock_runner.run.return_value = ("test error", 1)
+        self.mock_runner.run.return_value = ("stdout", "test error", 1)
 
         assert_raise_library_error(
             lambda: lib.qdevice_setup(self.mock_runner),
@@ -82,7 +82,7 @@ class QdeviceSetupTest(TestCase):
                 report_codes.QDEVICE_INITIALIZATION_ERROR,
                 {
                     "model": "net",
-                    "reason": "test error",
+                    "reason": "test error\nstdout",
                 }
             )
         )
@@ -126,7 +126,7 @@ class QdeviceStatusGenericTest(TestCase):
         self.mock_runner = mock.MagicMock(spec_set=CommandRunner)
 
     def test_success(self):
-        self.mock_runner.run.return_value = ("status info", 0)
+        self.mock_runner.run.return_value = ("status info", "", 0)
         self.assertEqual(
             "status info",
             lib.qdevice_status_generic_text(self.mock_runner)
@@ -134,7 +134,7 @@ class QdeviceStatusGenericTest(TestCase):
         self.mock_runner.run.assert_called_once_with([_qnetd_tool, "-s"])
 
     def test_success_verbose(self):
-        self.mock_runner.run.return_value = ("status info", 0)
+        self.mock_runner.run.return_value = ("status info", "", 0)
         self.assertEqual(
             "status info",
             lib.qdevice_status_generic_text(self.mock_runner, True)
@@ -142,7 +142,7 @@ class QdeviceStatusGenericTest(TestCase):
         self.mock_runner.run.assert_called_once_with([_qnetd_tool, "-s", "-v"])
 
     def test_error(self):
-        self.mock_runner.run.return_value = ("status error", 1)
+        self.mock_runner.run.return_value = ("some info", "status error", 1)
         assert_raise_library_error(
             lambda: lib.qdevice_status_generic_text(self.mock_runner),
             (
@@ -150,7 +150,7 @@ class QdeviceStatusGenericTest(TestCase):
                 report_codes.QDEVICE_GET_STATUS_ERROR,
                 {
                     "model": "net",
-                    "reason": "status error",
+                    "reason": "status error\nsome info",
                 }
             )
         )
@@ -162,7 +162,7 @@ class QdeviceStatusClusterTest(TestCase):
         self.mock_runner = mock.MagicMock(spec_set=CommandRunner)
 
     def test_success(self):
-        self.mock_runner.run.return_value = ("status info", 0)
+        self.mock_runner.run.return_value = ("status info", "", 0)
         self.assertEqual(
             "status info",
             lib.qdevice_status_cluster_text(self.mock_runner)
@@ -170,7 +170,7 @@ class QdeviceStatusClusterTest(TestCase):
         self.mock_runner.run.assert_called_once_with([_qnetd_tool, "-l"])
 
     def test_success_verbose(self):
-        self.mock_runner.run.return_value = ("status info", 0)
+        self.mock_runner.run.return_value = ("status info", "", 0)
         self.assertEqual(
             "status info",
             lib.qdevice_status_cluster_text(self.mock_runner, verbose=True)
@@ -178,7 +178,7 @@ class QdeviceStatusClusterTest(TestCase):
         self.mock_runner.run.assert_called_once_with([_qnetd_tool, "-l", "-v"])
 
     def test_success_cluster(self):
-        self.mock_runner.run.return_value = ("status info", 0)
+        self.mock_runner.run.return_value = ("status info", "", 0)
         self.assertEqual(
             "status info",
             lib.qdevice_status_cluster_text(self.mock_runner, "cluster")
@@ -188,7 +188,7 @@ class QdeviceStatusClusterTest(TestCase):
         ])
 
     def test_success_cluster_verbose(self):
-        self.mock_runner.run.return_value = ("status info", 0)
+        self.mock_runner.run.return_value = ("status info", "", 0)
         self.assertEqual(
             "status info",
             lib.qdevice_status_cluster_text(self.mock_runner, "cluster", True)
@@ -198,7 +198,7 @@ class QdeviceStatusClusterTest(TestCase):
         ])
 
     def test_error(self):
-        self.mock_runner.run.return_value = ("status error", 1)
+        self.mock_runner.run.return_value = ("some info", "status error", 1)
         assert_raise_library_error(
             lambda: lib.qdevice_status_cluster_text(self.mock_runner),
             (
@@ -206,13 +206,63 @@ class QdeviceStatusClusterTest(TestCase):
                 report_codes.QDEVICE_GET_STATUS_ERROR,
                 {
                     "model": "net",
-                    "reason": "status error",
+                    "reason": "status error\nsome info",
                 }
             )
         )
         self.mock_runner.run.assert_called_once_with([_qnetd_tool, "-l"])
 
 
+class QdeviceConnectedClustersTest(TestCase):
+    def test_empty_status(self):
+        status = ""
+        self.assertEqual(
+            [],
+            lib.qdevice_connected_clusters(status)
+        )
+
+    def test_one_cluster(self):
+        status = """\
+Cluster "rhel72":
+    Algorithm:          LMS
+    Tie-breaker:        Node with lowest node ID
+    Node ID 2:
+        Client address:         ::ffff:192.168.122.122:59738
+        Configured node list:   1, 2
+        Membership node list:   1, 2
+        Vote:                   ACK (ACK)
+    Node ID 1:
+        Client address:         ::ffff:192.168.122.121:43420
+        Configured node list:   1, 2
+        Membership node list:   1, 2
+        Vote:                   ACK (ACK)
+"""
+        self.assertEqual(
+            ["rhel72"],
+            lib.qdevice_connected_clusters(status)
+        )
+
+    def test_more_clusters(self):
+        status = """\
+Cluster "rhel72":
+Cluster "rhel73":
+"""
+        self.assertEqual(
+            ["rhel72", "rhel73"],
+            lib.qdevice_connected_clusters(status)
+        )
+
+    def test_invalid_status(self):
+        status = """\
+Cluster:
+    Cluster "rhel72":
+"""
+        self.assertEqual(
+            [],
+            lib.qdevice_connected_clusters(status)
+        )
+
+
 @mock.patch("pcs.lib.corosync.qdevice_net._get_output_certificate")
 @mock.patch("pcs.lib.corosync.qdevice_net._store_to_tmpfile")
 class QdeviceSignCertificateRequestTest(CertificateTestCase):
@@ -222,7 +272,7 @@ class QdeviceSignCertificateRequestTest(CertificateTestCase):
     )
     def test_success(self, mock_tmp_store, mock_get_cert):
         mock_tmp_store.return_value = self.mock_tmpfile
-        self.mock_runner.run.return_value = ("tool output", 0)
+        self.mock_runner.run.return_value = ("tool output", "", 0)
         mock_get_cert.return_value = "new certificate".encode("utf-8")
 
         result = lib.qdevice_sign_certificate_request(
@@ -293,7 +343,7 @@ class QdeviceSignCertificateRequestTest(CertificateTestCase):
     )
     def test_sign_error(self, mock_tmp_store, mock_get_cert):
         mock_tmp_store.return_value = self.mock_tmpfile
-        self.mock_runner.run.return_value = ("tool output error", 1)
+        self.mock_runner.run.return_value = ("stdout", "tool output error", 1)
 
         assert_raise_library_error(
             lambda: lib.qdevice_sign_certificate_request(
@@ -305,7 +355,7 @@ class QdeviceSignCertificateRequestTest(CertificateTestCase):
                 severity.ERROR,
                 report_codes.QDEVICE_CERTIFICATE_SIGN_ERROR,
                 {
-                    "reason": "tool output error",
+                    "reason": "tool output error\nstdout",
                 }
             )
         )
@@ -326,7 +376,7 @@ class QdeviceSignCertificateRequestTest(CertificateTestCase):
     )
     def test_output_read_error(self, mock_tmp_store, mock_get_cert):
         mock_tmp_store.return_value = self.mock_tmpfile
-        self.mock_runner.run.return_value = ("tool output", 0)
+        self.mock_runner.run.return_value = ("tool output", "", 0)
         mock_get_cert.side_effect = LibraryError
 
         self.assertRaises(
@@ -399,7 +449,7 @@ class ClientSetupTest(TestCase):
 
     @mock.patch("pcs.lib.corosync.qdevice_net.client_destroy")
     def test_success(self, mock_destroy):
-        self.mock_runner.run.return_value = ("tool output", 0)
+        self.mock_runner.run.return_value = ("tool output", "", 0)
 
         lib.client_setup(self.mock_runner, "certificate data".encode("utf-8"))
 
@@ -414,7 +464,7 @@ class ClientSetupTest(TestCase):
 
     @mock.patch("pcs.lib.corosync.qdevice_net.client_destroy")
     def test_init_error(self, mock_destroy):
-        self.mock_runner.run.return_value = ("tool output error", 1)
+        self.mock_runner.run.return_value = ("stdout", "tool output error", 1)
 
         assert_raise_library_error(
             lambda: lib.client_setup(
@@ -426,7 +476,7 @@ class ClientSetupTest(TestCase):
                 report_codes.QDEVICE_INITIALIZATION_ERROR,
                 {
                     "model": "net",
-                    "reason": "tool output error",
+                    "reason": "tool output error\nstdout",
                 }
             )
         )
@@ -448,7 +498,7 @@ class ClientGenerateCertificateRequestTest(CertificateTestCase):
         lambda: True
     )
     def test_success(self, mock_get_cert):
-        self.mock_runner.run.return_value = ("tool output", 0)
+        self.mock_runner.run.return_value = ("tool output", "", 0)
         mock_get_cert.return_value = "new certificate".encode("utf-8")
 
         result = lib.client_generate_certificate_request(
@@ -492,7 +542,7 @@ class ClientGenerateCertificateRequestTest(CertificateTestCase):
         lambda: True
     )
     def test_tool_error(self, mock_get_cert):
-        self.mock_runner.run.return_value = ("tool output error", 1)
+        self.mock_runner.run.return_value = ("stdout", "tool output error", 1)
 
         assert_raise_library_error(
             lambda: lib.client_generate_certificate_request(
@@ -504,7 +554,7 @@ class ClientGenerateCertificateRequestTest(CertificateTestCase):
                 report_codes.QDEVICE_INITIALIZATION_ERROR,
                 {
                     "model": "net",
-                    "reason": "tool output error",
+                    "reason": "tool output error\nstdout",
                 }
             )
         )
@@ -523,7 +573,7 @@ class ClientCertRequestToPk12Test(CertificateTestCase):
     )
     def test_success(self, mock_tmp_store, mock_get_cert):
         mock_tmp_store.return_value = self.mock_tmpfile
-        self.mock_runner.run.return_value = ("tool output", 0)
+        self.mock_runner.run.return_value = ("tool output", "", 0)
         mock_get_cert.return_value = "new certificate".encode("utf-8")
 
         result = lib.client_cert_request_to_pk12(
@@ -594,7 +644,7 @@ class ClientCertRequestToPk12Test(CertificateTestCase):
     )
     def test_transform_error(self, mock_tmp_store, mock_get_cert):
         mock_tmp_store.return_value = self.mock_tmpfile
-        self.mock_runner.run.return_value = ("tool output error", 1)
+        self.mock_runner.run.return_value = ("stdout", "tool output error", 1)
 
         assert_raise_library_error(
             lambda: lib.client_cert_request_to_pk12(
@@ -605,7 +655,7 @@ class ClientCertRequestToPk12Test(CertificateTestCase):
                 severity.ERROR,
                 report_codes.QDEVICE_CERTIFICATE_IMPORT_ERROR,
                 {
-                    "reason": "tool output error",
+                    "reason": "tool output error\nstdout",
                 }
             )
         )
@@ -625,7 +675,7 @@ class ClientCertRequestToPk12Test(CertificateTestCase):
     )
     def test_output_read_error(self, mock_tmp_store, mock_get_cert):
         mock_tmp_store.return_value = self.mock_tmpfile
-        self.mock_runner.run.return_value = ("tool output", 0)
+        self.mock_runner.run.return_value = ("tool output", "", 0)
         mock_get_cert.side_effect = LibraryError
 
         self.assertRaises(
@@ -657,7 +707,7 @@ class ClientImportCertificateAndKeyTest(CertificateTestCase):
     )
     def test_success(self, mock_tmp_store):
         mock_tmp_store.return_value = self.mock_tmpfile
-        self.mock_runner.run.return_value = ("tool output", 0)
+        self.mock_runner.run.return_value = ("tool output", "", 0)
 
         lib.client_import_certificate_and_key(
             self.mock_runner,
@@ -721,7 +771,7 @@ class ClientImportCertificateAndKeyTest(CertificateTestCase):
     )
     def test_import_error(self, mock_tmp_store):
         mock_tmp_store.return_value = self.mock_tmpfile
-        self.mock_runner.run.return_value = ("tool output error", 1)
+        self.mock_runner.run.return_value = ("stdout", "tool output error", 1)
 
         assert_raise_library_error(
             lambda: lib.client_import_certificate_and_key(
@@ -732,7 +782,7 @@ class ClientImportCertificateAndKeyTest(CertificateTestCase):
                 severity.ERROR,
                 report_codes.QDEVICE_CERTIFICATE_IMPORT_ERROR,
                 {
-                    "reason": "tool output error",
+                    "reason": "tool output error\nstdout",
                 }
             )
         )
