@@ -6,9 +6,9 @@ from __future__ import (
 )
 
 import json
-from unittest import TestCase
+from pcs.test.tools.pcs_unittest import TestCase
 
-from pcs.test.tools.pcs_mock import mock
+from pcs.test.tools.pcs_unittest import mock
 from pcs.test.tools.assertions import (
     assert_raise_library_error,
     assert_report_item_list_equal,
@@ -28,6 +28,7 @@ from pcs.lib.external import (
     NodeConnectionException,
 )
 import pcs.lib.sbd as lib_sbd
+from pcs.lib.corosync.config_facade import ConfigFacade as CorosyncConfigFacade
 
 
 class TestException(Exception):
@@ -50,11 +51,9 @@ class RunParallelAndRaiseLibErrorOnFailureTest(TestCase):
                 raise LibraryError(
                     ReportItem.error(
                         report_codes.COMMON_ERROR,
-                        "another report"
                     ),
                     ReportItem.info(
                         report_codes.COMMON_INFO,
-                        "just info"
                     )
                 )
 
@@ -85,6 +84,111 @@ class RunParallelAndRaiseLibErrorOnFailureTest(TestCase):
         )
 
 
+class EvenNumberOfNodesAndNoQdevice(TestCase):
+    def setUp(self):
+        self.mock_corosync_conf = mock.MagicMock(spec_set=CorosyncConfigFacade)
+
+    def _set_ret_vals(self, nodes, qdevice):
+        self.mock_corosync_conf.get_nodes.return_value = nodes
+        self.mock_corosync_conf.has_quorum_device.return_value = qdevice
+
+    def test_even_num_no_qdevice(self):
+        self._set_ret_vals([1, 2], False)
+        self.assertTrue(lib_sbd._even_number_of_nodes_and_no_qdevice(
+            self.mock_corosync_conf
+        ))
+
+    def test_even_num_qdevice(self):
+        self._set_ret_vals([1, 2], True)
+        self.assertFalse(lib_sbd._even_number_of_nodes_and_no_qdevice(
+            self.mock_corosync_conf
+        ))
+
+    def test_odd_num_no_qdevice(self):
+        self._set_ret_vals([1, 2, 3], False)
+        self.assertFalse(lib_sbd._even_number_of_nodes_and_no_qdevice(
+            self.mock_corosync_conf
+        ))
+
+    def test_odd_num_qdevice(self):
+        self._set_ret_vals([1, 2, 3], True)
+        self.assertFalse(lib_sbd._even_number_of_nodes_and_no_qdevice(
+            self.mock_corosync_conf
+        ))
+
+    def test_even_num_no_qdevice_plus_one(self):
+        self._set_ret_vals([1, 2], False)
+        self.assertFalse(lib_sbd._even_number_of_nodes_and_no_qdevice(
+            self.mock_corosync_conf, 1
+        ))
+
+    def test_even_num_qdevice_plus_one(self):
+        self._set_ret_vals([1, 2], True)
+        self.assertFalse(lib_sbd._even_number_of_nodes_and_no_qdevice(
+            self.mock_corosync_conf, 1
+        ))
+
+    def test_odd_num_no_qdevice_plus_one(self):
+        self._set_ret_vals([1, 2, 3], False)
+        self.assertTrue(lib_sbd._even_number_of_nodes_and_no_qdevice(
+            self.mock_corosync_conf, 1
+        ))
+
+    def test_odd_num_qdevice_plus_one(self):
+        self._set_ret_vals([1, 2, 3], True)
+        self.assertFalse(lib_sbd._even_number_of_nodes_and_no_qdevice(
+            self.mock_corosync_conf, 1
+        ))
+
+
+@mock.patch("pcs.lib.sbd.is_auto_tie_breaker_needed")
+class AtbHasToBeEnabledTest(TestCase):
+    def setUp(self):
+        self.mock_runner = "runner"
+        self.mock_conf = mock.MagicMock(spec_set=CorosyncConfigFacade)
+
+    def test_atb_needed_is_enabled(self, mock_is_needed):
+        mock_is_needed.return_value = True
+        self.mock_conf.is_enabled_auto_tie_breaker.return_value = True
+        self.assertFalse(lib_sbd.atb_has_to_be_enabled(
+            self.mock_runner, self.mock_conf, 1
+        ))
+        self.mock_conf.is_enabled_auto_tie_breaker.assert_called_once_with()
+        mock_is_needed.assert_not_called()
+
+    def test_atb_needed_is_disabled(self, mock_is_needed):
+        mock_is_needed.return_value = True
+        self.mock_conf.is_enabled_auto_tie_breaker.return_value = False
+        self.assertTrue(lib_sbd.atb_has_to_be_enabled(
+            self.mock_runner, self.mock_conf, -1
+        ))
+        self.mock_conf.is_enabled_auto_tie_breaker.assert_called_once_with()
+        mock_is_needed.assert_called_once_with(
+            self.mock_runner, self.mock_conf, -1
+        )
+
+    def test_atb_not_needed_is_enabled(self, mock_is_needed):
+        mock_is_needed.return_value = False
+        self.mock_conf.is_enabled_auto_tie_breaker.return_value = True
+        self.assertFalse(lib_sbd.atb_has_to_be_enabled(
+            self.mock_runner, self.mock_conf, 2
+        ))
+        self.mock_conf.is_enabled_auto_tie_breaker.assert_called_once_with()
+        mock_is_needed.assert_not_called()
+
+    def test_atb_not_needed_is_disabled(self, mock_is_needed):
+        mock_is_needed.return_value = False
+        self.mock_conf.is_enabled_auto_tie_breaker.return_value = False
+        self.assertFalse(lib_sbd.atb_has_to_be_enabled(
+            self.mock_runner, self.mock_conf, -2
+        ))
+        self.mock_conf.is_enabled_auto_tie_breaker.assert_called_once_with()
+        mock_is_needed.assert_called_once_with(
+            self.mock_runner, self.mock_conf, -2
+        )
+
+
+
 class CheckSbdTest(TestCase):
     def test_success(self):
         mock_communicator = mock.MagicMock(spec_set=NodeCommunicator)
@@ -316,11 +420,11 @@ class SetSbdConfigOnNodeTest(TestCase):
         }
         cfg_out = """# This file has been generated by pcs.
 SBD_OPTS="-n node1"
-SBD_WATCHDOG_DEV=/dev/watchdog
+SBD_WATCHDOG_DEV=/my/watchdog
 SBD_WATCHDOG_TIMEOUT=0
 """
         lib_sbd.set_sbd_config_on_node(
-            self.mock_rep, self.mock_com, self.node, cfg_in
+            self.mock_rep, self.mock_com, self.node, cfg_in, "/my/watchdog"
         )
         mock_set_sbd_cfg.assert_called_once_with(
             self.mock_com, self.node, cfg_out
@@ -340,17 +444,24 @@ class SetSbdConfigOnAllNodesTest(TestCase):
     def test_success(self, mock_func):
         mock_com = mock.MagicMock(spec_set=NodeCommunicator)
         mock_rep = MockLibraryReportProcessor()
-        node_list = [NodeAddresses("node" + str(i)) for i in range(5)]
+        watchdog_dict = dict([
+            (NodeAddresses("node" + str(i)), "/dev/watchdog" + str(i))
+            for i in range(5)
+        ])
+        node_list = list(watchdog_dict.keys())
         config = {
             "opt1": "val1",
             "opt2": "val2"
         }
         lib_sbd.set_sbd_config_on_all_nodes(
-            mock_rep, mock_com, node_list, config
+            mock_rep, mock_com, node_list, config, watchdog_dict
         )
         mock_func.assert_called_once_with(
             lib_sbd.set_sbd_config_on_node,
-            [([mock_rep, mock_com, node, config], {}) for node in node_list]
+            [
+                ([mock_rep, mock_com, node, config, watchdog_dict[node]], {})
+                for node in node_list
+            ]
         )
 
 
@@ -588,9 +699,47 @@ class GetSbdConfigTest(TestCase):
         )
 
 
+@mock.patch("pcs.lib.external.is_systemctl")
+class GetSbdServiceNameTest(TestCase):
+    def test_systemctl(self, mock_is_systemctl):
+        mock_is_systemctl.return_value = True
+        self.assertEqual("sbd", lib_sbd.get_sbd_service_name())
+        mock_is_systemctl.assert_called_once_with()
+
+    def test_not_systemctl(self, mock_is_systemctl):
+        mock_is_systemctl.return_value = False
+        self.assertEqual("sbd_helper", lib_sbd.get_sbd_service_name())
+        mock_is_systemctl.assert_called_once_with()
+
+
+@mock.patch("pcs.lib.sbd.get_sbd_service_name")
 @mock.patch("pcs.lib.external.is_service_enabled")
 class IsSbdEnabledTest(TestCase):
-    def test_success(self, mock_is_service_enabled):
+    def test_success(self, mock_is_service_enabled, mock_sbd_name):
         mock_obj = mock.MagicMock()
         mock_is_service_enabled.return_value = True
+        mock_sbd_name.return_value = "sbd"
         self.assertTrue(lib_sbd.is_sbd_enabled(mock_obj))
+        mock_is_service_enabled.assert_called_once_with(mock_obj, "sbd")
+        mock_sbd_name.assert_called_once_with()
+
+
+@mock.patch("pcs.lib.sbd.get_sbd_service_name")
+@mock.patch("pcs.lib.external.is_service_installed")
+class IsSbdInstalledTest(TestCase):
+    def test_installed(self, mock_is_service_installed, mock_sbd_name):
+        mock_obj = mock.MagicMock()
+        mock_is_service_installed.return_value = True
+        mock_sbd_name.return_value = "sbd"
+        self.assertTrue(lib_sbd.is_sbd_installed(mock_obj))
+        mock_is_service_installed.assert_called_once_with(mock_obj, "sbd")
+        mock_sbd_name.assert_called_once_with()
+
+    def test_not_installed(self, mock_is_service_installed, mock_sbd_name):
+        mock_obj = mock.MagicMock()
+        mock_is_service_installed.return_value = False
+        mock_sbd_name.return_value = "sbd"
+        self.assertFalse(lib_sbd.is_sbd_installed(mock_obj))
+        mock_is_service_installed.assert_called_once_with(mock_obj, "sbd")
+        mock_sbd_name.assert_called_once_with()
+
