@@ -6,7 +6,7 @@ from __future__ import (
 )
 
 import sys
-import unittest
+from pcs.test.tools import pcs_unittest as unittest
 import xml.dom.minidom
 import xml.etree.cElementTree as ET
 from time import sleep
@@ -273,6 +273,9 @@ class UtilsTest(unittest.TestCase):
                             name="remote-node" value="guest2"/>
                     </instance_attributes>
                 </primitive>
+                <primitive id="dummy3"
+                        class="ocf" provider="pacemaker" type="remote">
+                </primitive>
             </resources>
         """).documentElement
         resources = dom.getElementsByTagName("resources")[0]
@@ -296,6 +299,12 @@ class UtilsTest(unittest.TestCase):
                 utils.dom_get_resource(dom, "vm-guest1")
             )
         )
+        self.assertEqual(
+            "dummy3",
+            utils.dom_get_resource_remote_node_name(
+                utils.dom_get_resource(dom, "dummy3")
+            )
+        )
 
     def test_dom_get_meta_attr_value(self):
         dom = self.get_cib_empty()
@@ -1391,12 +1400,12 @@ class UtilsTest(unittest.TestCase):
         """).documentElement
         self.assertRaises(
             SystemExit,
-            utils.dom_update_utilization, el, [("name", "invalid_val")]
+            utils.dom_update_utilization, el, {"name": "invalid_val"}
         )
 
         self.assertRaises(
             SystemExit,
-            utils.dom_update_utilization, el, [("name", "0.01")]
+            utils.dom_update_utilization, el, {"name": "0.01"}
         )
 
         sys.stderr = tmp_stderr
@@ -1406,7 +1415,12 @@ class UtilsTest(unittest.TestCase):
         <resource id="test_id"/>
         """).documentElement
         utils.dom_update_utilization(
-            el, [("name", ""), ("key", "-1"), ("keys", "90")]
+            el,
+            {
+                "name": "",
+                "key": "-1",
+                "keys": "90",
+            }
         )
 
         self.assertEqual(len(dom_get_child_elements(el)), 1)
@@ -1450,7 +1464,11 @@ class UtilsTest(unittest.TestCase):
         </resource>
         """).documentElement
         utils.dom_update_utilization(
-            el, [("key", "100"), ("keys", "")]
+            el,
+            {
+                "key": "100",
+                "keys": "",
+            }
         )
 
         u = dom_get_child_elements(el)[0]
@@ -1797,7 +1815,10 @@ class RunParallelTest(unittest.TestCase):
             wait_seconds=.1
         )
 
-        self.assertEqual(log, ['first', 'second'])
+        self.assertEqual(
+            sorted(log),
+            sorted(['first', 'second'])
+        )
 
     def test_wait_for_slower_workers(self):
         log = []
