@@ -21,6 +21,7 @@ def full_usage():
     out += strip_extras(acl([],False))
     out += strip_extras(qdevice([],False))
     out += strip_extras(quorum([],False))
+    out += strip_extras(booth([],False))
     out += strip_extras(status([],False))
     out += strip_extras(config([],False))
     out += strip_extras(pcsd([],False))
@@ -117,6 +118,7 @@ def generate_completion_tree_from_usage():
     tree["pcsd"] = generate_tree(pcsd([],False))
     tree["node"] = generate_tree(node([], False))
     tree["alert"] = generate_tree(alert([], False))
+    tree["booth"] = generate_tree(booth([], False))
     return tree
 
 def generate_tree(usage_txt):
@@ -167,6 +169,7 @@ Commands:
     acl         Set pacemaker access control lists.
     qdevice     Manage quorum device provider.
     quorum      Manage cluster quorum settings.
+    booth       Manage booth (cluster ticket manager).
     status      View cluster status.
     config      View and manage cluster configuration.
     pcsd        Manage pcs daemon.
@@ -187,22 +190,22 @@ Usage: pcs resource [commands]...
 Manage pacemaker resources
 
 Commands:
-    [show [resource id]] [--full] [--groups]
+    [show [<resource id>] | --full | --groups | --hide-inactive]
         Show all currently configured resources or if a resource is specified
-        show the options for the configured resource.  If --full is specified
+        show the options for the configured resource.  If --full is specified,
         all configured resource options will be displayed.  If --groups is
-        specified, only show groups (and their resources).
+        specified, only show groups (and their resources).  If --hide-inactive
+        is specified, only show active resources.
 
+    list [filter] [--nodesc]
+        Show list of all available resource agents (if filter is provided then
+        only resource agents matching the filter will be shown). If --nodesc is
+        used then descriptions of resource agents are not printed.
 
-    list [<standard|provider|type>] [--nodesc]
-        Show list of all available resources, optionally filtered by specified
-        type, standard or provider.  If --nodesc is used then descriptions
-        of resources are not printed.
-
-    describe <standard:provider:type|type>
+    describe [<standard>:[<provider>:]]<type>
         Show options for the specified resource.
 
-    create <resource id> <standard:provider:type|type> [resource options]
+    create <resource id> [<standard>:[<provider>:]]<type> [resource options]
            [op <operation action> <operation options> [<operation action>
            <operation options>]...] [meta <meta options>...]
            [--clone <clone options> | --master <master options> |
@@ -681,6 +684,7 @@ Commands:
         the whole CIB or be warned in the case of outdated CIB.
 
     node add <node[,node-altaddr]> [--start [--wait[=<n>]]] [--enable]
+            [--watchdog=<watchdog-path>]
         Add the node to corosync.conf and corosync on all nodes in the cluster
         and sync the new corosync.conf to the new node.  If --start is
         specified also start corosync/pacemaker on the new node, if --wait is
@@ -688,6 +692,8 @@ Commands:
         is specified enable corosync/pacemaker on new node.
         When using Redundant Ring Protocol (RRP) with udpu transport, specify
         the ring 0 address first followed by a ',' and then the ring 1 address.
+        Use --watchdog to specify path to watchdog on newly added node, when SBD
+        is enabled in cluster.
 
     node remove <node>
         Shutdown specified node and remove it from pacemaker and corosync on
@@ -814,6 +820,10 @@ Commands:
         To prevent accidental running of this command, --force or interactive
         user response is required in order to proceed.
 
+        NOTE: It is not checked if the specified node exists in the cluster
+        in order to be able to work with nodes not visible from the local
+        cluster partition.
+
     sbd enable [--watchdog=<path>[@<node>]] ... [<SBD_OPTION>=<value>] ...
         Enable SBD in cluster. Default path for watchdog device is
         /dev/watchdog. Allowed SBD options: SBD_WATCHDOG_TIMEOUT (default: 5),
@@ -990,21 +1000,24 @@ Commands:
         List all current ticket constraints (if --full is specified show
         the internal constraint id's as well).
 
-    ticket add <ticket> [<role>] <resource id> [options]
-               [id=constraint-id]
+    ticket add <ticket> [<role>] <resource id> [<options>]
+               [id=<constraint-id>]
         Create a ticket constraint for <resource id>.
         Available option is loss-policy=fence/stop/freeze/demote.
         A role can be master, slave, started or stopped.
 
-    ticket set <resource1> [resourceN]... [options]
-               [set <resourceX> ... [options]]
-               [setoptions [constraint_options]]
+    ticket set <resource1> [<resourceN>]... [<options>]
+               [set <resourceX> ... [<options>]]
+               setoptions <constraint_options>
         Create a ticket constraint with a resource set.
         Available options are sequential=true/false, require-all=true/false,
         action=start/promote/demote/stop and role=Stopped/Started/Master/Slave.
         Required constraint option is ticket=<ticket>. Optional constraint
         options are id=<constraint-id> and loss-policy=fence/stop/freeze/demote.
 
+    ticket remove <ticket> <resource id>
+        Remove all ticket constraints with <ticket> from <resource id>.
+
     remove [constraint id]...
         Remove constraint(s) or constraint rules with the specified id(s).
 
@@ -1063,21 +1076,26 @@ Commands:
         Delete the role specified and remove it from any users/groups it was
         assigned to.
 
-    role assign <role id> [to] <username/group>
+    role assign <role id> [to] [user|group] <username/group>
         Assign a role to a user or group already created with 'pcs acl
-        user/group create'.
+        user/group create'. If there is user and group with the same id and it
+        is not specified which should be used, user will be prioritized. In
+        cases like this specify whenever user or group should be used.
 
-    role unassign <role id> [from] <username/group>
-        Remove a role from the specified user.
+    role unassign <role id> [from] [user|group] <username/group>
+        Remove a role from the specified user. If there is user and group with
+        the same id and it is not specified which should be used, user will be
+        prioritized. In cases like this specify whenever user or group should
+        be used.
 
-    user create <username> <role id> [<role id>]...
+    user create <username> [<role id>]...
         Create an ACL for the user specified and assign roles to the user.
 
     user delete <username>
         Remove the user specified (and roles assigned will be unassigned for
         the specified user).
 
-    group create <group> <role id> [<role id>]...
+    group create <group> [<role id>]...
         Create an ACL for the group specified and assign roles to the group.
 
     group delete <group>
@@ -1106,8 +1124,12 @@ Commands:
         View all information about the cluster and resources (--full provides
         more details, --hide-inactive hides inactive resources).
 
-    resources
-        View current status of cluster resources.
+    resources [<resource id> | --full | --groups | --hide-inactive]
+        Show all currently configured resources or if a resource is specified
+        show the options for the configured resource.  If --full is specified,
+        all configured resource options will be displayed.  If --groups is
+        specified, only show groups (and their resources).  If --hide-inactive
+        is specified, only show active resources.
 
     groups
         View currently configured groups and their resources.
@@ -1118,6 +1140,14 @@ Commands:
     corosync
         View current membership information as seen by corosync.
 
+    quorum
+        View current quorum status.
+
+    qdevice <device model> [--full] [<cluster name>]
+        Show runtime status of specified model of quorum device provider.  Using
+        --full will give more detailed output.  If <cluster name> is specified,
+        only information about the specified cluster will be displayed.
+
     nodes [corosync|both|config]
         View current status of nodes from pacemaker. If 'corosync' is
         specified, print nodes currently configured in corosync, if 'both'
@@ -1165,7 +1195,7 @@ Commands:
         Restore cluster configuration to specified checkpoint.
 
     import-cman output=<filename> [input=<filename>] [--interactive]
-            [output-format=corosync.conf|cluster.conf]
+            [output-format=corosync.conf|cluster.conf] [dist=<dist>]
         Converts CMAN cluster configuration to Pacemaker cluster configuration.
         Converted configuration will be saved to 'output' file.  To send
         the configuration to the cluster nodes the 'pcs config restore'
@@ -1173,20 +1203,30 @@ Commands:
         prompted to solve incompatibilities manually.  If no input is specified
         /etc/cluster/cluster.conf will be used.  You can force to create output
         containing either cluster.conf or corosync.conf using the output-format
-        option.
+        option.  Optionally you can specify output version by setting 'dist'
+        option e. g. rhel,6.8 or redhat,7.3 or debian,7 or ubuntu,trusty.  You
+        can get the list of supported dist values by running the "clufter
+        --list-dists" command.  If 'dist' is not specified, it defaults to this
+        node's version if that matches output-format, otherwise redhat,6.7 is
+        used for cluster.conf and redhat,7.1 is used for corosync.conf.
 
     import-cman output=<filename> [input=<filename>] [--interactive]
-            output-format=pcs-commands|pcs-commands-verbose
+            output-format=pcs-commands|pcs-commands-verbose [dist=<dist>]
         Converts CMAN cluster configuration to a list of pcs commands which
         recreates the same cluster as Pacemaker cluster when executed.  Commands
         will be saved to 'output' file.  For other options see above.
 
-    export pcs-commands|pcs-commands-verbose output=<filename>
+    export pcs-commands|pcs-commands-verbose [output=<filename>] [dist=<dist>]
         Creates a list of pcs commands which upon execution recreates
         the current cluster running on this node.  Commands will be saved
-        to 'output' file.  Use pcs-commands to get a simple list of commands,
-        whereas pcs-commands-verbose creates a list including comments and debug
-        messages.
+        to 'output' file or written to stdout if 'output' is not specified.  Use
+        pcs-commands to get a simple list of commands, whereas
+        pcs-commands-verbose creates a list including comments and debug
+        messages.  Optionally specify output version by setting 'dist' option
+        e. g. rhel,6.8 or redhat,7.3 or debian,7 or ubuntu,trusty.  You can get
+        the list of supported dist values by running the "clufter --list-dists"
+        command.  If 'dist' is not specified, it defaults to this node's
+        version.
 """
     if pout:
         print(sub_usage(args, output))
@@ -1227,6 +1267,14 @@ Usage: pcs node <command>
 Manage cluster nodes
 
 Commands:
+    attribute [[<node>] [--name <name>] | <node> <name>=<value> ...]
+        Manage node attributes.  If no parameters are specified, show attributes
+        of all nodes.  If one parameter is specified, show attributes
+        of specified node.  If --name is specified, show specified attribute's
+        value from all nodes.  If more parameters are specified, set attributes
+        of specified node.  Attributes can be removed by setting an attribute
+        without a value.
+
     maintenance [--all] | [<node>]...
         Put specified node(s) into maintenance mode, if no node or options are
         specified the current node will be put into maintenance mode, if --all
@@ -1257,12 +1305,13 @@ Commands:
         the operation not succeeded yet.  If 'n' is not specified it defaults
         to 60 minutes.
 
-    utilization [<node> [<name>=<value> ...]]
-        Add specified utilization options to specified node. If node is not
-        specified, shows utilization of all nodes. If utilization options are
-        not specified, shows utilization of specified node. Utilization option
-        should be in format name=value, value has to be integer. Options may be
-        removed by setting an option without a value.
+    utilization [[<node>] [--name <name>] | <node> <name>=<value> ...]
+        Add specified utilization options to specified node.  If node is not
+        specified, shows utilization of all nodes.  If --name is specified,
+        shows specified utilization value from all nodes. If utilization options
+        are not specified, shows utilization of specified node.  Utilization
+        option should be in format name=value, value has to be integer.  Options
+        may be removed by setting an option without a value.
         Example: pcs node utilization node1 cpu=4 ram=
 """
     if pout:
@@ -1322,7 +1371,7 @@ Usage: pcs quorum <command>
 Manage cluster quorum settings.
 
 Commands:
-    config
+    [config]
         Show quorum configuration.
 
     status
@@ -1380,6 +1429,82 @@ Commands:
     else:
         return output
 
+def booth(args=[], pout=True):
+    output = """
+Usage: pcs booth <command>
+Manage booth (cluster ticket manager)
+
+Commands:
+    setup sites <address> <address> [<address>...] [arbitrators <address> ...]
+            [--force]
+        Write new booth configuration with specified sites and arbitrators.
+        Total number of peers (sites and arbitrators) must be odd.  When
+        the configuration file already exists, command fails unless --force
+        is specified.
+
+    destroy
+        Remove booth configuration files.
+
+    ticket add <ticket> [<name>=<value> ...]
+        Add new ticket to the current configuration. Ticket options are
+        specified in booth manpage.
+
+    ticket remove <ticket>
+        Remove the specified ticket from the current configuration.
+
+    config [<node>]
+        Show booth configuration from the specified node or from the current
+        node if node not specified.
+
+    create ip <address>
+        Make the cluster run booth service on the specified ip address as
+        a cluster resource.  Typically this is used to run booth site.
+
+    remove
+        Remove booth resources created by the "pcs booth create" command.
+
+    restart
+        Restart booth resources created by the "pcs booth create" command.
+
+    ticket grant <ticket> [<site address>]
+        Grant the ticket for the site specified by address.  Site address which
+        has been specified with 'pcs booth create' command is used if
+        'site address' is omitted.  Specifying site address is mandatory when
+        running this command on an arbitrator.
+
+    ticket revoke <ticket> [<site address>]
+        Revoke the ticket for the site specified by address.  Site address which
+        has been specified with 'pcs booth create' command is used if
+        'site address' is omitted.  Specifying site address is mandatory when
+        running this command on an arbitrator.
+
+    status
+        Print current status of booth on the local node.
+
+    pull <node>
+        Pull booth configuration from the specified node.
+
+    sync [--skip-offline]
+        Send booth configuration from the local node to all nodes
+        in the cluster.
+
+    enable
+        Enable booth arbitrator service.
+
+    disable
+        Disable booth arbitrator service.
+
+    start
+        Start booth arbitrator service.
+
+    stop
+        Stop booth arbitrator service.
+"""
+    if pout:
+        print(sub_usage(args, output))
+    else:
+        return output
+
 
 def alert(args=[], pout=True):
     output = """
@@ -1392,26 +1517,28 @@ Commands:
 
     create path=<path> [id=<alert-id>] [description=<description>]
             [options [<option>=<value>]...] [meta [<meta-option>=<value>]...]
-        Create new alert with specified path. Id will be automatically
+        Define an alert handler with specified path. Id will be automatically
         generated if it is not specified.
 
     update <alert-id> [path=<path>] [description=<description>]
             [options [<option>=<value>]...] [meta [<meta-option>=<value>]...]
-        Update existing alert with specified id.
+        Update existing alert handler with specified id.
 
-    remove <alert-id>
-        Remove alert with specified id.
+    remove <alert-id> ...
+        Remove alert handlers with specified ids.
 
-    recipient add <alert-id> <recipient-value> [description=<description>]
-            [options [<option>=<value>]...] [meta [<meta-option>=<value>]...]
-        Add new recipient to specified alert.
+    recipient add <alert-id> value=<recipient-value> [id=<recipient-id>]
+            [description=<description>] [options [<option>=<value>]...]
+            [meta [<meta-option>=<value>]...]
+        Add new recipient to specified alert handler.
 
-    recipient update <alert-id> <recipient-value> [description=<description>]
-            [options [<option>=<value>]...] [meta [<meta-option>=<value>]...]
-        Update existing recipient identified by alert and it's value.
+    recipient update <recipient-id> [value=<recipient-value>]
+            [description=<description>] [options [<option>=<value>]...]
+            [meta [<meta-option>=<value>]...]
+        Update existing recipient identified by it's id.
 
-    recipient remove <alert-id> <recipient-value>
-        Remove specified recipient.
+    recipient remove <recipient-id> ...
+        Remove specified recipients.
 """
     if pout:
         print(sub_usage(args, output))
@@ -1431,6 +1558,7 @@ def show(main_usage_name, rest_usage_names):
         "property": property,
         "qdevice": qdevice,
         "quorum": quorum,
+        "booth": booth,
         "resource": resource,
         "status": status,
         "stonith": stonith,
