@@ -25,6 +25,51 @@ import base64
 import threading
 import logging
 
+
+from pcs import settings, usage
+from pcs.cli.common.reports import (
+    process_library_reports,
+    LibraryReportProcessorToConsole as LibraryReportProcessorToConsole,
+)
+from pcs.common.tools import (
+    join_multilines,
+    simple_cache,
+)
+from pcs.lib import reports, sbd
+from pcs.lib.env import LibraryEnvironment
+from pcs.lib.errors import LibraryError
+from pcs.lib.external import (
+    CommandRunner,
+    disable_service,
+    DisableServiceError,
+    enable_service,
+    EnableServiceError,
+    is_cman_cluster as lib_is_cman_cluster,
+    is_service_enabled,
+    is_service_running,
+    is_systemctl,
+    _service,
+    _systemctl,
+)
+import pcs.lib.resource_agent as lib_ra
+import pcs.lib.corosync.config_parser as corosync_conf_parser
+from pcs.lib.corosync.config_facade import ConfigFacade as corosync_conf_facade
+from pcs.lib.pacemaker import has_resource_wait_support
+from pcs.lib.pacemaker_state import ClusterState
+from pcs.lib.pacemaker_values import(
+    validate_id,
+    is_boolean,
+    timeout_to_seconds as get_timeout_seconds,
+    is_score_value,
+)
+from pcs.cli.common import middleware
+from pcs.cli.common.env import Env
+from pcs.cli.common.lib_wrapper import Library
+from pcs.cli.common.reports import build_report_message
+from pcs.cli.booth.command import DEFAULT_BOOTH_NAME
+import pcs.cli.booth.env
+
+
 try:
     # python2
     from urllib import urlencode as urllib_urlencode
@@ -55,49 +100,16 @@ except ImportError:
     )
 
 
-from pcs import settings, usage
-from pcs.cli.common.reports import (
-    process_library_reports,
-    LibraryReportProcessorToConsole as LibraryReportProcessorToConsole,
-)
-from pcs.common.tools import simple_cache
-from pcs.lib import reports
-from pcs.lib.env import LibraryEnvironment
-from pcs.lib.errors import LibraryError
-from pcs.lib.external import (
-    CommandRunner,
-    is_cman_cluster,
-    is_systemctl,
-    is_service_enabled,
-    is_service_running,
-    disable_service,
-    DisableServiceError,
-    enable_service,
-    EnableServiceError,
-)
-import pcs.lib.resource_agent as lib_ra
-import pcs.lib.corosync.config_parser as corosync_conf_parser
-from pcs.lib.corosync.config_facade import ConfigFacade as corosync_conf_facade
-from pcs.lib.pacemaker import has_resource_wait_support
-from pcs.lib.pacemaker_state import ClusterState
-from pcs.lib.pacemaker_values import(
-    validate_id,
-    is_boolean,
-    timeout_to_seconds as get_timeout_seconds,
-    is_score_value,
-)
-from pcs.cli.common import middleware
-from pcs.cli.common.env import Env
-from pcs.cli.common.lib_wrapper import Library
 
 
 PYTHON2 = sys.version[0] == "2"
 
+DEFAULT_RESOURCE_ACTIONS = ["monitor", "start", "stop", "promote", "demote"]
+
 # usefile & filename variables are set in pcs module
 usefile = False
 filename = ""
 pcs_options = {}
-fence_bin = settings.fence_agent_binaries
 
 
 class UnknownPropertyException(Exception):
@@ -301,6 +313,8 @@ def canAddNodeToCluster(node):
                 return (False, "unable to authenticate to node")
             if "node_available" in myout and myout["node_available"] == True:
                 return (True, "")
+            elif myout.get("pacemaker_remote", False):
+                return (False, "node is running pacemaker_remote")
             else:
                 return (False, "node is already in a cluster")
         except ValueError:
@@ -384,7 +398,8 @@ def sendHTTPRequest(host, request, data = None, printResult = True, printSuccess
                 if "CIB_user" == name:
                     value = re.sub(r"[^!-~]", "", value).replace(";", "")
                 else:
-                    value = base64.b64encode(value)
+                    # python3 requires the value to be bytes not str
+                    value = base64.b64encode(value.encode("utf8"))
                 cookies.append("{0}={1}".format(name, value))
     if cookies:
         opener.addheaders.append(('Cookie', ";".join(cookies)))
@@ -465,6 +480,24 @@ def getNodesFromPacemaker():
     except LibraryError as e:
         process_library_reports(e.args)
 
+def getNodeAttributesFromPacemaker():
+    try:
+        return [
+            node.attrs
+            for node in ClusterState(getClusterStateXml()).node_section.nodes
+        ]
+    except LibraryError as e:
+        process_library_reports(e.args)
+
+
+def hasCorosyncConf(conf=None):
+    if not conf:
+        if is_rhel6():
+            conf = settings.cluster_conf_file
+        else:
+            conf = settings.corosync_conf_file
+    return os.path.isfile(conf)
+
 def getCorosyncConf(conf=None):
     if not conf:
         if is_rhel6():
@@ -559,6 +592,23 @@ def getCorosyncActiveNodes():
 
     return nodes_active
 
+
+def _enable_auto_tie_breaker_for_sbd(corosync_conf):
+    """
+    Enable auto tie breaker in specified corosync conf if it is needed by SBD.
+
+    corosync_conf -- parsed corosync conf
+    """
+    try:
+        corosync_facade = corosync_conf_facade(corosync_conf)
+        if sbd.atb_has_to_be_enabled(cmd_runner(), corosync_facade):
+            corosync_facade.set_quorum_options(
+                get_report_processor(), {"auto_tie_breaker": "1"}
+            )
+    except LibraryError as e:
+        process_library_reports(e.args)
+
+
 # Add node specified to corosync.conf and reload corosync.conf (if running)
 def addNodeToCorosync(node):
 # Before adding, make sure node isn't already in corosync.conf
@@ -585,6 +635,9 @@ def addNodeToCorosync(node):
         new_node.add_attribute("ring1_addr", node1)
     new_node.add_attribute("nodeid", new_nodeid)
 
+    # enable ATB if it's needed
+    _enable_auto_tie_breaker_for_sbd(corosync_conf)
+
     corosync_conf = autoset_2node_corosync(corosync_conf)
     setCorosyncConf(str(corosync_conf))
     return True
@@ -612,12 +665,40 @@ def addNodeToClusterConf(node):
                 "error adding alternative address for node: %s" % node0
             )
 
+    # ensure the pacemaker fence device exists
+    pcmk_fence_name = None
+    all_fence_names = set()
+    output, retval = run([
+        "ccs", "-i", "-f", settings.cluster_conf_file, "--lsfencedev"
+    ])
+    if retval == 0:
+        for line in output.splitlines():
+            fence_name, dummy_fence_args = line.split(":", 1)
+            all_fence_names.add(fence_name)
+            match = re.match("(^|(.* ))agent=fence_pcmk((,.+)|$)", line)
+            if match:
+                pcmk_fence_name = fence_name
+    if not pcmk_fence_name:
+        fence_index = 1
+        pcmk_fence_name = "pcmk-redirect"
+        while pcmk_fence_name in all_fence_names:
+            pcmk_fence_name = "pcmk-redirect-{0}".format(fence_index)
+            fence_index += 1
+
+        output, retval = run([
+            "ccs", "-i", "-f", settings.cluster_conf_file,
+            "--addfencedev", pcmk_fence_name, "agent=fence_pcmk",
+        ])
+        if retval != 0:
+            print(output)
+            err("error fence device for node: %s" % node)
+
     output, retval = run(["ccs", "-i", "-f", settings.cluster_conf_file, "--addmethod", "pcmk-method", node0])
     if retval != 0:
         print(output)
         err("error adding fence method: %s" % node)
 
-    output, retval = run(["ccs", "-i", "-f", settings.cluster_conf_file, "--addfenceinst", "pcmk-redirect", node0, "pcmk-method", "port="+node0])
+    output, retval = run(["ccs", "-i", "-f", settings.cluster_conf_file, "--addfenceinst", pcmk_fence_name, node0, "pcmk-method", "port="+node0])
     if retval != 0:
         print(output)
         err("error adding fence instance: %s" % node)
@@ -652,6 +733,9 @@ def removeNodeFromCorosync(node):
                     removed_node = True
 
     if removed_node:
+        # enable ATB if it's needed
+        _enable_auto_tie_breaker_for_sbd(corosync_conf)
+
         corosync_conf = autoset_2node_corosync(corosync_conf)
         setCorosyncConf(str(corosync_conf))
 
@@ -792,6 +876,7 @@ def run(
         env_extend = dict()
     env_var = env_extend
     env_var.update(dict(os.environ))
+    env_var["LC_ALL"] = "C"
     if usefile:
         env_var["CIB_file"] = filename
 
@@ -852,6 +937,7 @@ def cmd_runner():
     if usefile:
         env_vars["CIB_file"] = filename
     env_vars.update(os.environ)
+    env_vars["LC_ALL"] = "C"
     return CommandRunner(
         logging.getLogger("old_cli"),
         get_report_processor(),
@@ -1033,16 +1119,16 @@ def create_task_list(report, action, node_list, *args, **kwargs):
     ]
 
 def parallel_for_nodes(action, node_list, *args, **kwargs):
-    error_list = []
+    node_errors = dict()
     def report(node, returncode, output):
         message = '{0}: {1}'.format(node, output.strip())
         print(message)
         if returncode != 0:
-            error_list.append(message)
+            node_errors[node] = message
     run_parallel(
         create_task_list(report, action, node_list, *args, **kwargs)
     )
-    return error_list
+    return node_errors
 
 def prepare_node_name(node, pm_nodes, cs_nodes):
     '''
@@ -1071,18 +1157,6 @@ def does_exist(xpath_query):
         return False
     return True
 
-def is_pacemaker_node(node):
-    p_nodes = getNodesFromPacemaker()
-    if node in p_nodes:
-        return True
-    return False
-
-def is_corosync_node(node):
-    c_nodes = getNodesFromCorosyncConf()
-    if node in c_nodes:
-        return True
-    return False
-
 def get_group_children(group_id):
     child_resources = []
     dom = get_cib_dom()
@@ -1254,6 +1328,14 @@ def validate_constraint_resource(dom, resource_id):
 def dom_get_resource_remote_node_name(dom_resource):
     if dom_resource.tagName != "primitive":
         return None
+    if (
+        dom_resource.getAttribute("class").lower() == "ocf"
+        and
+        dom_resource.getAttribute("provider").lower() == "pacemaker"
+        and
+        dom_resource.getAttribute("type").lower() == "remote"
+    ):
+        return dom_resource.getAttribute("id")
     return dom_get_meta_attr_value(dom_resource, "remote-node")
 
 def dom_get_meta_attr_value(dom_resource, meta_name):
@@ -1395,37 +1477,47 @@ def resource_running_on(resource, passed_state=None, stopped=False):
         "nodes_slave": nodes_slave,
     }
 
-def does_resource_have_options(ra_type):
-    if ra_type.startswith("ocf:") or ra_type.startswith("stonith:") or ra_type.find(':') == -1:
-        return True
-    return False
+def filter_default_op_from_actions(resource_actions):
+    filtered = []
+    for action in resource_actions:
+        if action.get("name", "") not in DEFAULT_RESOURCE_ACTIONS:
+            continue
+        new_action = dict([
+            (name, value)
+            for name, value in action.items()
+            if name != "depth"
+        ])
+        filtered.append(new_action)
+    return filtered
 
 # Given a resource agent (ocf:heartbeat:XXX) return an list of default
 # operations or an empty list if unable to find any default operations
-def get_default_op_values(ra_type):
-    allowable_operations = ["monitor", "start", "stop", "promote", "demote"]
+def get_default_op_values(full_agent_name):
     default_ops = []
     try:
-        metadata = lib_ra.get_resource_agent_metadata(cmd_runner(), ra_type)
-        actions = lib_ra.get_agent_actions(metadata)
+        if full_agent_name.startswith("stonith:"):
+            metadata = lib_ra.StonithAgent(
+                cmd_runner(),
+                full_agent_name[len("stonith:"):]
+            )
+        else:
+            metadata = lib_ra.ResourceAgent(
+                cmd_runner(),
+                full_agent_name
+            )
+        actions = filter_default_op_from_actions(metadata.get_actions())
 
         for action in actions:
-            if action["name"] not in allowable_operations:
-                continue
             op = [action["name"]]
             for key in action.keys():
-                if key != "name" and (key != "depth" or action[key] != "0"):
+                if key != "name" and action[key] != "0":
                     op.append("{0}={1}".format(key, action[key]))
             default_ops.append(op)
-    except (
-        lib_ra.UnsupportedResourceAgent,
-        lib_ra.AgentNotFound,
-        lib_ra.UnableToGetAgentMetadata
-    ):
+    except lib_ra.UnableToGetAgentMetadata:
         return []
-    except lib_ra.ResourceAgentLibError as e:
+    except lib_ra.ResourceAgentError as e:
         process_library_reports(
-            [lib_ra.resource_agent_lib_error_to_report_item(e)]
+            [lib_ra.resource_agent_error_to_report_item(e)]
         )
     except LibraryError as e:
         process_library_reports(e.args)
@@ -1454,67 +1546,6 @@ def validate_wait_get_timeout(need_cib_support=True):
     return wait_timeout
 
 
-def is_file_abs_path(path):
-    return path == os.path.abspath(path) and os.path.isfile(path)
-
-# Check and see if the specified resource (or stonith) type is present on the
-# file system and properly responds to a meta-data request
-def is_valid_resource(resource, caseInsensitiveCheck=False):
-    try:
-        if resource.startswith("stonith:"):
-            lib_ra.get_fence_agent_metadata(
-                cmd_runner(), resource.split("stonith:", 1)[1]
-            )
-        else:
-            lib_ra.get_resource_agent_metadata(cmd_runner(), resource)
-        # return True if no exception was raised
-        return True
-    except lib_ra.UnsupportedResourceAgent:
-        pass
-    except (lib_ra.ResourceAgentLibError, LibraryError):
-        # agent not exists or obtaining metadata failed
-        return False
-
-    if resource.startswith("lsb:"):
-        agent = os.path.join("/etc/init.d/", resource.split(":", 1)[1])
-        return is_file_abs_path(agent)
-    elif resource.startswith("systemd:"):
-        _, agent_name = resource.split(":", 1)
-        agent1 = os.path.join(
-            "/etc/systemd/system/", agent_name + ".service"
-        )
-        agent2 = os.path.join(
-            "/usr/lib/systemd/system/", agent_name + ".service"
-        )
-        return is_file_abs_path(agent1) or is_file_abs_path(agent2)
-
-    # resource name is not full, maybe it's ocf resource
-    for provider in sorted(os.listdir(settings.ocf_resources)):
-        provider_path = os.path.join(settings.ocf_resources, provider)
-        if caseInsensitiveCheck:
-            if os.path.isdir(provider_path):
-                for f in os.listdir(provider_path):
-                    if (
-                        f.lower() == resource.lower() and
-                        os.path.isfile(os.path.join(provider_path, f))
-                    ):
-                        return "ocf:{0}:{1}".format(provider, f)
-                continue
-
-        if os.path.exists(
-            os.path.join(settings.ocf_resources, provider, resource)
-        ):
-            try:
-                lib_ra.get_resource_agent_metadata(
-                    cmd_runner(),
-                    "ocf:{0}:{1}".format(provider, resource)
-                )
-                return True
-            except (LibraryError, lib_ra.ResourceAgentLibError):
-                continue
-    return False
-
-
 # Return matches from the CIB with the xpath_query
 def get_cib_xpath(xpath_query):
     args = ["cibadmin", "-Q", "--xpath", xpath_query]
@@ -1591,15 +1622,35 @@ def is_valid_cib_scope(scope):
 # Checks to see if id exists in the xml dom passed
 # DEPRECATED use lxml version available in pcs.lib.cib.tools
 def does_id_exist(dom, check_id):
+    # do not search in /cib/status, it may contain references to previously
+    # existing and deleted resources and thus preventing creating them again
     if is_etree(dom):
-        for elem in dom.findall(str(".//*")):
+        for elem in dom.findall(str(
+            '(/cib/*[name()!="status"]|/*[name()!="cib"])/*'
+        )):
             if elem.get("id") == check_id:
                 return True
     else:
-        all_elem = dom.getElementsByTagName("*")
-        for elem in all_elem:
-            if elem.getAttribute("id") == check_id:
-                return True
+        document = (
+            dom
+            if isinstance(dom, xml.dom.minidom.Document)
+            else dom.ownerDocument
+        )
+        cib_found = False
+        for cib in dom_get_children_by_tag_name(document, "cib"):
+            cib_found = True
+            for section in cib.childNodes:
+                if section.nodeType != xml.dom.minidom.Node.ELEMENT_NODE:
+                    continue
+                if section.tagName == "status":
+                    continue
+                for elem in section.getElementsByTagName("*"):
+                    if elem.getAttribute("id") == check_id:
+                        return True
+        if not cib_found:
+            for elem in document.getElementsByTagName("*"):
+                if elem.getAttribute("id") == check_id:
+                    return True
     return False
 
 # Returns check_id if it doesn't exist in the dom, otherwise it adds an integer
@@ -1661,19 +1712,26 @@ def set_unmanaged(resource):
             "is-managed", "--meta", "--parameter-value", "false"]
     return run(args)
 
-def get_node_attributes():
+def get_node_attributes(filter_node=None, filter_attr=None):
     node_config = get_cib_xpath("//nodes")
-    nas = {}
     if (node_config == ""):
         err("unable to get crm_config, is pacemaker running?")
     dom = parseString(node_config).documentElement
+    nas = dict()
     for node in dom.getElementsByTagName("node"):
         nodename = node.getAttribute("uname")
+        if filter_node is not None and nodename != filter_node:
+            continue
         for attributes in node.getElementsByTagName("instance_attributes"):
             for nvp in attributes.getElementsByTagName("nvpair"):
+                attr_name = nvp.getAttribute("name")
+                if filter_attr is not None and attr_name != filter_attr:
+                    continue
                 if nodename not in nas:
-                    nas[nodename] = []
-                nas[nodename].append(nvp.getAttribute("name") + "=" + nvp.getAttribute("value"))
+                    nas[nodename] = dict()
+                nas[nodename][attr_name] = nvp.getAttribute("value")
+            # Use just first element of attributes. We don't support
+            # attributes with rules just yet.
             break
     return nas
 
@@ -1727,7 +1785,7 @@ def set_cib_property(prop, value, cib_dom=None):
     if update_cib:
         replace_cib_configuration(crm_config)
 
-def setAttribute(a_type, a_name, a_value):
+def setAttribute(a_type, a_name, a_value, exit_on_error=False):
     args = ["crm_attribute", "--type", a_type, "--attr-name", a_name,
             "--attr-value", a_value]
 
@@ -1736,7 +1794,10 @@ def setAttribute(a_type, a_name, a_value):
 
     output, retval = run(args)
     if retval != 0:
-        print(output)
+        if exit_on_error:
+            err(output)
+        else:
+            print(output)
 
 def getTerminalSize(fd=1):
     """
@@ -1763,14 +1824,24 @@ def get_terminal_input(message=None):
     if message:
         sys.stdout.write(message)
         sys.stdout.flush()
-    if PYTHON2:
-        return raw_input("")
-    else:
-        return input("")
+    try:
+        if PYTHON2:
+            return raw_input("")
+        else:
+            return input("")
+    except EOFError:
+        return ""
+    except KeyboardInterrupt:
+        print("Interrupted")
+        sys.exit(1)
 
 def get_terminal_password(message="Password: "):
-    if sys.stdout.isatty():
-        return getpass.getpass(message)
+    if sys.stdin.isatty():
+        try:
+            return getpass.getpass(message)
+        except KeyboardInterrupt:
+            print("Interrupted")
+            sys.exit(1)
     else:
         return get_terminal_input(message)
 
@@ -1798,22 +1869,26 @@ def stonithCheck():
                         prop.attrib["value"] == "false":
                     return False
 
-    primitives = et.findall(str("configuration/resources/primitive"))
-    for p in primitives:
-        if p.attrib["class"] == "stonith":
-            return False
+    xpath_list = (
+        "configuration/resources/primitive",
+        "configuration/resources/group/primitive",
+        "configuration/resources/clone/primitive",
+        "configuration/resources/clone/group/primitive",
+        "configuration/resources/master/primitive",
+        "configuration/resources/master/group/primitive",
+    )
+    for xpath in xpath_list:
+        for p in et.findall(str(xpath)):
+            if ("class" in p.attrib) and (p.attrib["class"] == "stonith"):
+                return False
 
-    primitives = et.findall(str("configuration/resources/clone/primitive"))
-    for p in primitives:
-        if p.attrib["class"] == "stonith":
-            return False
-
-    # check if SBD daemon is running
-    try:
-        if is_service_running(cmd_runner(), "sbd"):
-            return False
-    except LibraryError:
-        pass
+    if not usefile:
+        # check if SBD daemon is running
+        try:
+            if is_service_running(cmd_runner(), sbd.get_sbd_service_name()):
+                return False
+        except LibraryError:
+            pass
 
     return True
 
@@ -1837,7 +1912,7 @@ def getCorosyncNodesID(allow_failure=False):
         err_msgs, retval, output, dummy_std_err = call_local_pcsd(
             ['status', 'nodes', 'corosync-id'], True
         )
-        if err_msgs:
+        if err_msgs and not allow_failure:
             for msg in err_msgs:
                 err(msg, False)
             sys.exit(1)
@@ -1851,18 +1926,21 @@ def getCorosyncNodesID(allow_failure=False):
     cs_nodes = {}
     node_list_node_mapping = {}
     for line in output.rstrip().split("\n"):
-        m = re.match("nodelist.node.(\d+).nodeid.*= (.*)",line)
+        m = re.match("nodelist\.node\.(\d+)\.nodeid.*= (.*)", line)
         if m:
             node_list_node_mapping[m.group(1)] = m.group(2)
 
     for line in output.rstrip().split("\n"):
-        m = re.match("nodelist.node.(\d+).ring0_addr.*= (.*)",line)
-        if m:
+        m = re.match("nodelist\.node\.(\d+)\.ring0_addr.*= (.*)", line)
+        # check if node id is in node_list_node_mapping - do not crash when
+        # node ids are not specified
+        if m and m.group(1) in node_list_node_mapping:
             cs_nodes[node_list_node_mapping[m.group(1)]] = m.group(2)
     return cs_nodes
 
 # Warning, if a node has never started the hostname may be '(null)'
 #TODO This doesn't work on CMAN clusters at all and should be removed completely
+# Doesn't work on pacemaker-remote nodes either
 def getPacemakerNodesID(allow_failure=False):
     if os.getuid() == 0:
         (output, retval) = run(['crm_node', '-l'])
@@ -1870,7 +1948,7 @@ def getPacemakerNodesID(allow_failure=False):
         err_msgs, retval, output, dummy_std_err = call_local_pcsd(
             ['status', 'nodes', 'pacemaker-id'], True
         )
-        if err_msgs:
+        if err_msgs and not allow_failure:
             for msg in err_msgs:
                 err(msg, False)
             sys.exit(1)
@@ -1890,9 +1968,11 @@ def getPacemakerNodesID(allow_failure=False):
     return pm_nodes
 
 def corosyncPacemakerNodeCheck():
-    # does not work on CMAN clusters
-    pm_nodes = getPacemakerNodesID()
-    cs_nodes = getCorosyncNodesID()
+    # does not work on CMAN clusters and pacemaker-remote nodes
+    # we do not want a failure to exit pcs as this is only a minor information
+    # function
+    pm_nodes = getPacemakerNodesID(allow_failure=True)
+    cs_nodes = getCorosyncNodesID(allow_failure=True)
 
     for node_id in pm_nodes:
         if pm_nodes[node_id] == "(null)":
@@ -1917,10 +1997,9 @@ def getClusterName():
     if is_rhel6():
         try:
             dom = parse(settings.cluster_conf_file)
+            return dom.documentElement.getAttribute("name")
         except (IOError,xml.parsers.expat.ExpatError):
-            return ""
-
-        return dom.documentElement.getAttribute("name")
+            pass
     else:
         try:
             f = open(settings.corosync_conf_file,'r')
@@ -1934,7 +2013,15 @@ def getClusterName():
             if cluster_name:
                 return cluster_name
         except (IOError, corosync_conf_parser.CorosyncConfParserException):
-            return ""
+            pass
+
+    # there is no corosync.conf or cluster.conf on remote nodes, we can try to
+    # get cluster name from pacemaker
+    try:
+        return get_set_properties("cluster-name")["cluster-name"]
+    except:
+        # we need to catch SystemExit (from utils.err), parse errors and so on
+        pass
 
     return ""
 
@@ -1969,7 +2056,7 @@ def validate_xml_id(var, description="id"):
     try:
         validate_id(var, description)
     except LibraryError as e:
-        return False, e.args[0].message
+        return False, build_report_message(e.args[0])
     return True, ""
 
 def is_iso8601_date(var):
@@ -2007,9 +2094,12 @@ def verify_cert_key_pair(cert, key):
     return errors
 
 
-@simple_cache
 def is_rhel6():
-    return is_cman_cluster(cmd_runner())
+    return is_cman_cluster()
+
+@simple_cache
+def is_cman_cluster():
+    return lib_is_cman_cluster(cmd_runner())
 
 def err(errorText, exit_after_error=True):
     sys.stderr.write("Error: %s\n" % errorText)
@@ -2018,26 +2108,32 @@ def err(errorText, exit_after_error=True):
 
 
 def serviceStatus(prefix):
-    if not is_systemctl():
-        return
     print("Daemon Status:")
-    for service in ["corosync", "pacemaker", "pcsd"]:
-        print('{0}{1}: {2}/{3}'.format(
-            prefix, service,
-            run(["systemctl", 'is-active', service])[0].strip(),
-            run(["systemctl", 'is-enabled', service])[0].strip()
-        ))
-    try:
-        sbd_running = is_service_running(cmd_runner(), "sbd")
-        sbd_enabled = is_service_enabled(cmd_runner(), "sbd")
-        if sbd_enabled or sbd_running:
-            print("{prefix}sbd: {active}/{enabled}".format(
-                prefix=prefix,
-                active=("active" if sbd_running else "inactive"),
-                enabled=("enabled" if sbd_enabled else "disabled")
-            ))
-    except LibraryError:
-        pass
+    service_def = [
+        # (
+        #     service name,
+        #     display even if not enabled nor running
+        # )
+        ("cman", False),
+        ("corosync", True),
+        ("pacemaker", True),
+        ("pacemaker_remote", False),
+        ("pcsd", True),
+        (sbd.get_sbd_service_name(), False),
+    ]
+    for service, display_always in service_def:
+        try:
+            running = is_service_running(cmd_runner(), service)
+            enabled = is_service_enabled(cmd_runner(), service)
+            if display_always or enabled or running:
+                print("{prefix}{service}: {active}/{enabled}".format(
+                    prefix=prefix,
+                    service=service,
+                    active=("active" if running else "inactive"),
+                    enabled=("enabled" if enabled else "disabled")
+                ))
+        except LibraryError:
+            pass
 
 def enableServices():
     # do NOT handle SBD in here, it is started by pacemaker not systemd or init
@@ -2078,6 +2174,22 @@ def disableServices():
     if report_item_list:
         raise LibraryError(*report_item_list)
 
+def start_service(service):
+    if is_systemctl():
+        stdout, stderr, retval = cmd_runner().run([
+            _systemctl, "start", service
+        ])
+    else:
+        stdout, stderr, retval = cmd_runner().run([_service, service, "start"])
+    return join_multilines([stderr, stdout]), retval
+
+def stop_service(service):
+    if is_systemctl():
+        stdout, stderr, retval = cmd_runner().run([_systemctl, "stop", service])
+    else:
+        stdout, stderr, retval = cmd_runner().run([_service, service, "stop"])
+    return join_multilines([stderr, stdout]), retval
+
 def write_file(path, data, permissions=0o644, binary=False):
     if os.path.exists(path):
         if "--force" not in pcs_options:
@@ -2398,7 +2510,7 @@ def dom_update_utilization(dom_element, attributes, id_prefix=""):
         id_prefix + dom_element.getAttribute("id") + "-utilization"
     )
 
-    for name, value in attributes:
+    for name, value in sorted(attributes.items()):
         if value != "" and not is_int(value):
             err(
                 "Value of utilization attribute must be integer: "
@@ -2426,21 +2538,22 @@ def dom_update_meta_attr(dom_element, attributes):
             meta_attributes.getAttribute("id") + "-"
         )
 
-def get_utilization(element):
+def get_utilization(element, filter_name=None):
     utilization = {}
     for e in element.getElementsByTagName("utilization"):
         for u in e.getElementsByTagName("nvpair"):
             name = u.getAttribute("name")
-            value = u.getAttribute("value") if u.hasAttribute("value") else ""
-            utilization[name] = value
+            if filter_name is not None and name != filter_name:
+                continue
+            utilization[name] = u.getAttribute("value")
         # Use just first element of utilization attributes. We don't support
         # utilization with rules just yet.
         break
     return utilization
 
-def get_utilization_str(element):
+def get_utilization_str(element, filter_name=None):
     output = []
-    for name, value in sorted(get_utilization(element).items()):
+    for name, value in sorted(get_utilization(element, filter_name).items()):
         output.append(name + "=" + value)
     return " ".join(output)
 
@@ -2518,33 +2631,44 @@ def get_cluster_properties_definition():
     ]
     definition = {}
     for source in sources:
-        output, retval = run([source["path"], "metadata"])
+        stdout, stderr, retval = cmd_runner().run([source["path"], "metadata"])
         if retval != 0:
-            err("unable to run {0}\n".format(source["name"]) + output)
-        etree = ET.fromstring(output)
-        for e in etree.findall("./parameters/parameter"):
-            prop = get_cluster_property_from_xml(e)
-            if prop["name"] not in banned_props:
-                prop["source"] = source["name"]
-                prop["advanced"] = prop["name"] not in basic_props
-                if prop["name"] in readable_names:
-                    prop["readable_name"] = readable_names[prop["name"]]
-                else:
-                    prop["readable_name"] = prop["name"]
-                definition[prop["name"]] = prop
+            err("unable to run {0}\n{1}".format(source["name"], stderr))
+        try:
+            etree = ET.fromstring(stdout)
+            for e in etree.findall("./parameters/parameter"):
+                prop = get_cluster_property_from_xml(e)
+                if prop["name"] not in banned_props:
+                    prop["source"] = source["name"]
+                    prop["advanced"] = prop["name"] not in basic_props
+                    if prop["name"] in readable_names:
+                        prop["readable_name"] = readable_names[prop["name"]]
+                    else:
+                        prop["readable_name"] = prop["name"]
+                    definition[prop["name"]] = prop
+        except xml.parsers.expat.ExpatError as e:
+            err("unable to parse {0} metadata definition: {1}".format(
+                source["name"],
+                e
+            ))
+        except ET.ParseError as e:
+            err("unable to parse {0} metadata definition: {1}".format(
+                source["name"],
+                e
+            ))
     return definition
 
 
 def get_cluster_property_from_xml(etree_el):
     property = {
-        "name": etree_el.get("name"),
-        "shortdesc": etree_el.find("shortdesc").text,
-        "longdesc": etree_el.find("longdesc").text
+        "name": etree_el.get("name", ""),
+        "shortdesc": "",
+        "longdesc": "",
     }
-    if property["shortdesc"] is None:
-        property["shortdesc"] = ""
-    if property["longdesc"] is None:
-        property["longdesc"] = ""
+    for item in ["shortdesc", "longdesc"]:
+        item_el = etree_el.find(item)
+        if item_el is not None and item_el.text is not None:
+            property[item] = item_el.text
 
     content = etree_el.find("content")
     if content is None:
@@ -2568,19 +2692,6 @@ def get_cluster_property_from_xml(etree_el):
         property["longdesc"] = ""
     return property
 
-# DEPRECATED use lxml version available in pcs.lib.cib.tools
-def get_acls(dom):
-    acls = dom.getElementsByTagName("acls")
-    if len(acls) == 0:
-        acls = dom.createElement("acls")
-        conf = dom.getElementsByTagName("configuration")
-        if len(conf) == 0:
-            err("Unable to get configuration section of cib")
-        conf[0].appendChild(acls)
-    else:
-        acls = acls[0]
-    return acls
-
 def get_lib_env():
     user = None
     groups = None
@@ -2639,7 +2750,15 @@ def get_middleware_factory():
         cib=middleware.cib(usefile, get_cib, replace_cib_configuration),
         corosync_conf_existing=middleware.corosync_conf_existing(
             pcs_options.get("--corosync_conf", None)
-        )
+        ),
+        booth_conf=pcs.cli.booth.env.middleware_config(
+            pcs_options.get("--name", DEFAULT_BOOTH_NAME),
+            pcs_options.get("--booth-conf", None),
+            pcs_options.get("--booth-key", None),
+        ),
+        cluster_conf_read_only=middleware.cluster_conf_read_only(
+            pcs_options.get("--cluster_conf", None)
+        ),
     )
 
 def get_library_wrapper():
@@ -2655,7 +2774,9 @@ def get_modificators():
     #commands is not an issue
     return {
         "autocorrect": "--autocorrect" in pcs_options,
+        "autodelete": "--autodelete" in pcs_options,
         "corosync_conf": pcs_options.get("--corosync_conf", None),
+        "describe": "--nodesc" not in pcs_options,
         "enable": "--enable" in pcs_options,
         "force": "--force" in pcs_options,
         "full": "--full" in pcs_options,
@@ -2674,3 +2795,16 @@ def exit_on_cmdline_input_errror(error, main_name, usage_name):
 
 def get_report_processor():
     return LibraryReportProcessorToConsole(debug=("--debug" in pcs_options))
+
+def get_set_properties(prop_name=None, defaults=None):
+    properties = {} if defaults is None else dict(defaults)
+    (output, retVal) = run(["cibadmin","-Q","--scope", "crm_config"])
+    if retVal != 0:
+        err("unable to get crm_config\n"+output)
+    dom = parseString(output)
+    de = dom.documentElement
+    crm_config_properties = de.getElementsByTagName("nvpair")
+    for prop in crm_config_properties:
+        if prop_name is None or (prop_name == prop.getAttribute("name")):
+            properties[prop.getAttribute("name")] = prop.getAttribute("value")
+    return properties
