@@ -120,7 +120,7 @@ module ClusterEntity
       status = ClusterEntity::CRMResourceStatus.new
       status.id = primitive.id
       status.resource_agent = primitive.agentname
-      status.managed = false
+      status.managed = true
       status.failed = resource[:failed]
       status.role = nil
       status.active = resource[:active]
@@ -332,7 +332,7 @@ module ClusterEntity
       :unknown => {
         :val => 6,
         :str => 'unknown'
-      }
+      },
     }
 
     def initialize(status=:unknown)
@@ -1000,7 +1000,7 @@ module ClusterEntity
   class Node < JSONable
     attr_accessor :id, :error_list, :warning_list, :status, :quorum, :uptime,
                   :name, :corosync, :pacemaker, :cman, :corosync_enabled,
-                  :pacemaker_enabled, :pcsd_enabled
+                  :pacemaker_enabled, :pcsd_enabled, :services, :sbd_config
 
     def initialize
       @id = nil
@@ -1010,22 +1010,38 @@ module ClusterEntity
       @quorum = nil
       @uptime = 'unknown'
       @name = nil
+      @services = {}
+      [
+        :pacemaker, :pacemaker_remote, :corosync, :pcsd, :cman, :sbd
+      ].each do |service|
+        @services[service] = {
+          :installed => nil,
+          :running => nil,
+          :enabled => nil
+        }
+      end
       @corosync = false
       @pacemaker = false
       @cman = false
       @corosync_enabled = false
       @pacemaker_enabled = false
       @pcsd_enabled = false
+      @sbd_config = nil
     end
 
     def self.load_current_node(crm_dom=nil)
       node = ClusterEntity::Node.new
-      node.corosync = corosync_running?
-      node.corosync_enabled = corosync_enabled?
-      node.pacemaker = pacemaker_running?
-      node.pacemaker_enabled = pacemaker_enabled?
-      node.cman = cman_running?
-      node.pcsd_enabled = pcsd_enabled?
+      node.services.each do |service, info|
+        info[:running] = is_service_running?(service.to_s)
+        info[:enabled] = is_service_enabled?(service.to_s)
+        info[:installed] = is_service_installed?(service.to_s)
+      end
+      node.corosync = node.services[:corosync][:running]
+      node.corosync_enabled = node.services[:corosync][:enabled]
+      node.pacemaker = node.services[:pacemaker][:running]
+      node.pacemaker_enabled = node.services[:pacemaker][:enabled]
+      node.cman = node.services[:cman][:running]
+      node.pcsd_enabled = node.services[:pcsd][:enabled]
 
       node_online = (node.corosync and node.pacemaker)
       node.status =  node_online ? 'online' : 'offline'
@@ -1044,7 +1060,7 @@ module ClusterEntity
       else
         node.status = 'offline'
       end
-
+      node.sbd_config = get_parsed_local_sbd_config()
       return node
     end
   end
