@@ -8,6 +8,7 @@ require 'net/https'
 require 'json'
 require 'fileutils'
 require 'backports'
+require 'base64'
 
 require 'config.rb'
 require 'cfgsync.rb'
@@ -19,6 +20,9 @@ require 'auth.rb'
 class NotImplementedException < NotImplementedError
 end
 
+class InvalidFileNameException < NameError
+end
+
 def getAllSettings(auth_user, cib_dom=nil)
   unless cib_dom
     cib_dom = get_cib_dom(auth_user)
@@ -131,6 +135,19 @@ def add_order_set_constraint(
   return retval, stderr.join(' ')
 end
 
+def add_colocation_set_constraint(
+  auth_user, resource_set_list, force=false, autocorrect=true
+)
+  command = [PCS, "constraint", "colocation"]
+  resource_set_list.each { |resource_set|
+    command << "set"
+    command.concat(resource_set)
+  }
+  command << '--force' if force
+  command << '--autocorrect' if autocorrect
+  stdout, stderr, retval = run_cmd(auth_user, *command)
+  return retval, stderr.join(' ')
+end
 
 def add_ticket_constraint(
     auth_user, ticket, resource_id, role, loss_policy,
@@ -231,12 +248,14 @@ def add_acl_usergroup(auth_user, acl_role_id, user_group, name)
     if retval == 0
       return ""
     end
-    if not /^error: (user|group) #{name.to_s} already exists$/i.match(stderr.join("\n").strip)
+    $logger.info(stdout)
+    if not /^Error: '#{name.to_s}' already exists$/i.match(stderr.join("\n").strip)
       return stderr.join("\n").strip
     end
   end
   stdout, stderror, retval = run_cmd(
-    auth_user, PCS, "acl", "role", "assign", acl_role_id.to_s, name.to_s
+    auth_user, PCS, "acl", "role", "assign",
+    acl_role_id.to_s, user_group, name.to_s
   )
   if retval != 0
     if stderror.empty?
@@ -262,11 +281,18 @@ def remove_acl_permission(auth_user, acl_perm_id)
   return ""
 end
 
-def remove_acl_usergroup(auth_user, role_id, usergroup_id)
-  stdout, stderror, retval = run_cmd(
-    auth_user, PCS, "acl", "role", "unassign", role_id.to_s, usergroup_id.to_s,
-    "--autodelete"
-  )
+def remove_acl_usergroup(auth_user, role_id, usergroup_id, user_or_group)
+  if ['user', 'group'].include?(user_or_group)
+    stdout, stderror, retval = run_cmd(
+      auth_user, PCS, "acl", "role", "unassign", role_id.to_s, user_or_group,
+      usergroup_id.to_s, "--autodelete"
+    )
+  else
+    stdout, stderror, retval = run_cmd(
+      auth_user, PCS, "acl", "role", "unassign", role_id.to_s,
+      usergroup_id.to_s, "--autodelete"
+    )
+  end
   if retval != 0
     if stderror.empty?
       return "Error removing user / group"
@@ -382,47 +408,47 @@ end
 
 def send_request(auth_user, node, request, post=false, data={}, remote=true, raw_data=nil, timeout=30, cookies_data=nil)
   cookies_data = {} if not cookies_data
+  request = "/#{request}" if not request.start_with?("/")
+
+  # fix ipv6 address for URI.parse
+  node6 = node
+  if (node.include?(":") and ! node.start_with?("["))
+    node6 = "[#{node}]"
+  end
+
+  if remote
+    uri = URI.parse("https://#{node6}:2224/remote" + request)
+  else
+    uri = URI.parse("https://#{node6}:2224" + request)
+  end
+
+  if post
+    req = Net::HTTP::Post.new(uri.path)
+    raw_data ? req.body = raw_data : req.set_form_data(data)
+  else
+    req = Net::HTTP::Get.new(uri.path)
+    req.set_form_data(data)
+  end
+
+  cookies_to_send = []
+  cookies_data_default = {}
+  # Let's be safe about characters in cookie variables and do base64.
+  # We cannot do it for CIB_user however to be backward compatible
+  # so we at least remove disallowed characters.
+  cookies_data_default['CIB_user'] = PCSAuth.cookieUserSafe(
+    auth_user[:username].to_s
+  )
+  cookies_data_default['CIB_user_groups'] = PCSAuth.cookieUserEncode(
+    (auth_user[:usergroups] || []).join(' ')
+  )
+
+  cookies_data_default.update(cookies_data)
+  cookies_data_default.each { |name, value|
+    cookies_to_send << CGI::Cookie.new('name' => name, 'value' => value).to_s
+  }
+  req.add_field('Cookie', cookies_to_send.join(';'))
+
   begin
-    request = "/#{request}" if not request.start_with?("/")
-
-    # fix ipv6 address for URI.parse
-    node6 = node
-    if (node.include?(":") and ! node.start_with?("["))
-      node6 = "[#{node}]"
-    end
-
-    if remote
-      uri = URI.parse("https://#{node6}:2224/remote" + request)
-    else
-      uri = URI.parse("https://#{node6}:2224" + request)
-    end
-
-    if post
-      req = Net::HTTP::Post.new(uri.path)
-      raw_data ? req.body = raw_data : req.set_form_data(data)
-    else
-      req = Net::HTTP::Get.new(uri.path)
-      req.set_form_data(data)
-    end
-
-    cookies_to_send = []
-    cookies_data_default = {}
-    # Let's be safe about characters in cookie variables and do base64.
-    # We cannot do it for CIB_user however to be backward compatible
-    # so we at least remove disallowed characters.
-    cookies_data_default['CIB_user'] = PCSAuth.cookieUserSafe(
-      auth_user[:username].to_s
-    )
-    cookies_data_default['CIB_user_groups'] = PCSAuth.cookieUserEncode(
-      (auth_user[:usergroups] || []).join(' ')
-    )
-
-    cookies_data_default.update(cookies_data)
-    cookies_data_default.each { |name, value|
-      cookies_to_send << CGI::Cookie.new('name' => name, 'value' => value).to_s
-    }
-    req.add_field('Cookie', cookies_to_send.join(';'))
-
     # uri.host returns "[addr]" for ipv6 addresses, which is wrong
     # uri.hostname returns "addr" for ipv6 addresses, which is correct, but it
     #   is not available in older ruby versions
@@ -879,6 +905,10 @@ def pacemaker_running?()
   is_service_running?('pacemaker')
 end
 
+def pacemaker_remote_running?()
+  is_service_running?('pacemaker_remote')
+end
+
 def get_pacemaker_version()
   begin
     stdout, stderror, retval = run_cmd(
@@ -928,9 +958,9 @@ def pcsd_restart()
   fork {
     sleep(10)
     if ISSYSTEMCTL
-      `systemctl restart pcsd`
+      exec("systemctl", "restart", "pcsd")
     else
-      `service pcsd restart`
+      exec("service", "pcsd", "restart")
     end
   }
 end
@@ -973,6 +1003,7 @@ def run_cmd_options(auth_user, options, *args)
     ps_write.fcntl(Fcntl::F_SETFD, Fcntl::FD_CLOEXEC)
     ENV['CIB_user'] = cib_user
     ENV['CIB_user_groups'] = cib_groups
+    ENV['LC_ALL'] = 'C'
     exec(*args)
   }
 
@@ -1340,10 +1371,10 @@ def pcsd_restart_nodes(auth_user, nodes)
   }
 end
 
-def write_file_lock(path, perm, data)
+def write_file_lock(path, perm, data, binary=false)
+  file = nil
   begin
-    file = nil
-    file = File.open(path, 'w', perm)
+    file = File.open(path, binary ? 'wb' : 'w', perm)
     file.flock(File::LOCK_EX)
     file.write(data)
   rescue => e
@@ -1357,6 +1388,23 @@ def write_file_lock(path, perm, data)
   end
 end
 
+def read_file_lock(path, binary=false)
+  file = nil
+  begin
+    file = File.open(path, binary ? 'rb' : 'r')
+    file.flock(File::LOCK_SH)
+    return file.read()
+  rescue => e
+    $logger.error("Cannot read file '#{path}': #{e.message}")
+    raise
+  ensure
+    unless file.nil?
+      file.flock(File::LOCK_UN)
+      file.close()
+    end
+  end
+end
+
 def verify_cert_key_pair(cert, key)
   errors = []
   cert_modulus = nil
@@ -1680,8 +1728,16 @@ def get_node_status(auth_user, cib_dom)
       :fence_levels => get_fence_levels(auth_user, cib_dom),
       :node_attr => node_attrs_to_v2(get_node_attributes(auth_user, cib_dom)),
       :nodes_utilization => get_nodes_utilization(cib_dom),
+      :alerts => get_alerts(auth_user),
       :known_nodes => [],
-      :available_features => ['sbd', 'ticket_constraints']
+      :available_features => [
+        'constraint_colocation_set',
+        'sbd',
+        'ticket_constraints',
+        'moving_resource_in_group',
+        'unmanaged_resource',
+        'alerts',
+      ]
   }
 
   nodes = get_nodes_status()
@@ -1820,7 +1876,7 @@ end
 def status_v1_to_v2(status)
   new_status = status.select { |k,_|
     [:cluster_name, :username, :is_cman_with_udpu_transport,
-     :need_ring1_address, :cluster_settings, :constraints, :groups,
+     :need_ring1_address, :cluster_settings, :constraints,
      :corosync_online, :corosync_offline, :pacemaker_online, :pacemaker_standby,
      :pacemaker_offline, :acls, :fence_levels
     ].include?(k)
@@ -1841,6 +1897,8 @@ def status_v1_to_v2(status)
     ].include?(k)
   }
 
+  new_status[:groups] = get_group_list_from_tree_of_resources(resources)
+
   new_status[:node].update(
     {
       :id => status[:node_id],
@@ -1857,6 +1915,22 @@ def status_v1_to_v2(status)
   return new_status
 end
 
+def get_group_list_from_tree_of_resources(tree)
+  group_list = []
+  tree.each { |resource|
+    if resource.instance_of?(ClusterEntity::Group)
+      group_list << resource.id
+    end
+    if (
+      resource.kind_of?(ClusterEntity::MultiInstance) and
+      resource.member.instance_of?(ClusterEntity::Group)
+    )
+      group_list << resource.member.id
+    end
+  }
+  return group_list
+end
+
 def allowed_for_local_cluster(auth_user, action)
   pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
   return pcs_config.permissions_local.allows?(
@@ -1921,17 +1995,18 @@ def is_service_installed?(service)
       if line.split(' ')[0] == service
         return true
       end
-      return false
     }
+    return false
   end
+
   stdout, _, retcode = run_cmd(
-    PCSAuth.getSuperuserAuth(), PCS, 'resource', 'list', 'systemd'
+    PCSAuth.getSuperuserAuth(), 'systemctl', 'list-unit-files', '--full'
   )
   if retcode != 0
     return nil
   end
   stdout.each { |line|
-    if line.strip() == "systemd:#{service}"
+    if line.strip().start_with?("#{service}.service")
       return true
     end
   }
@@ -1951,14 +2026,14 @@ def enable_service(service)
 end
 
 def disable_service(service)
+  # fails when the service is not installed, so we need to check it beforehand
+  if not is_service_installed?(service)
+    return true
+  end
+
   if ISSYSTEMCTL
-    # returns success even if the service is not installed
     cmd = ['systemctl', 'disable', "#{service}.service"]
   else
-    if not is_service_installed?(service)
-      return true
-    end
-    # fails when the service is not installed, so we need to check it beforehand
     cmd = ['chkconfig', service, 'off']
   end
   _, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), *cmd)
@@ -1966,9 +2041,12 @@ def disable_service(service)
 end
 
 def start_service(service)
-  _, _, retcode = run_cmd(
-    PCSAuth.getSuperuserAuth(), "service", service, "start"
-  )
+  if ISSYSTEMCTL
+    cmd = ['systemctl', 'start', "#{service}.service"]
+  else
+    cmd = ['service', service, 'start']
+  end
+  _, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), *cmd)
   return (retcode == 0)
 end
 
@@ -1976,9 +2054,12 @@ def stop_service(service)
   if not is_service_installed?(service)
     return true
   end
-  _, _, retcode = run_cmd(
-    PCSAuth.getSuperuserAuth(), "service", service, "stop"
-  )
+  if ISSYSTEMCTL
+    cmd = ['systemctl', 'stop', "#{service}.service"]
+  else
+    cmd = ['service', service, 'stop']
+  end
+  _, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), *cmd)
   return (retcode == 0)
 end
 
@@ -2005,3 +2086,74 @@ def get_parsed_local_sbd_config()
     return nil
   end
 end
+
+def get_sbd_service_name()
+  if ISSYSTEMCTL
+    return 'sbd'
+  else
+    return 'sbd_helper'
+  end
+end
+
+def write_booth_config(config, data)
+  if config.include?('/')
+    raise InvalidFileNameException.new(config)
+  end
+  write_file_lock(File.join(BOOTH_CONFIG_DIR, config), nil, data)
+end
+
+def read_booth_config(config)
+  if config.include?('/')
+    raise InvalidFileNameException.new(config)
+  end
+  config_path = File.join(BOOTH_CONFIG_DIR, config)
+  unless File.file?(config_path)
+    return nil
+  end
+  return read_file_lock(config_path)
+end
+
+def write_booth_authfile(filename, data)
+  if filename.include?('/')
+    raise InvalidFileNameException.new(filename)
+  end
+  write_file_lock(
+    File.join(BOOTH_CONFIG_DIR, filename), 0600, Base64.decode64(data), true
+  )
+end
+
+def read_booth_authfile(filename)
+  if filename.include?('/')
+    raise InvalidFileNameException.new(filename)
+  end
+  return Base64.strict_encode64(
+    read_file_lock(File.join(BOOTH_CONFIG_DIR, filename), true)
+  )
+end
+
+def get_authfile_from_booth_config(config_data)
+  authfile_path = nil
+  config_data.split("\n").each {|line|
+    if line.include?('=')
+      parts = line.split('=', 2)
+      if parts[0].strip == 'authfile'
+        authfile_path = parts[1].strip
+      end
+    end
+  }
+  return authfile_path
+end
+
+def get_alerts(auth_user)
+  out, _, retcode = run_cmd(auth_user, PCS, 'alert', 'get_all_alerts')
+
+  if retcode !=  0
+    return nil
+  end
+
+  begin
+    return JSON.parse(out.join(""))
+  rescue JSON::ParserError
+    return nil
+  end
+end
