@@ -96,50 +96,77 @@ function select_menu(menu, item, initial) {
 }
 
 function create_group() {
-  var num_nodes = 0;
-  var node_names = "";
-  $("#resource_list :checked").parent().parent().each(function (index,element) {
-    if (element.getAttribute("nodeID")) {
-      num_nodes++;
-      node_names += element.getAttribute("nodeID") + " "
-    }
-  });
-
-  if (num_nodes == 0) {
+  var resource_list = get_checked_ids_from_nodelist("resource_list");
+  if (resource_list.length == 0) {
     alert("You must select at least one resource to add to a group");
     return;
   }
-
-  $("#resources_to_add_to_group").val(node_names);
+  var not_primitives = resource_list.filter(function(resource_id) {
+    return !Pcs.resourcesContainer.get_resource_by_id(resource_id).get(
+      "is_primitive"
+    );
+  });
+  if (not_primitives.length != 0) {
+    alert("Members of group have to be primitive resources. These resources" +
+      " are not primitives: " + not_primitives.join(", "));
+    return;
+  }
+  var order_el = $("#new_group_resource_list tbody");
+  order_el.empty();
+  order_el.append(resource_list.map(function (item) {
+    return `<tr value="${item}" class="cursor-move"><td>${item}</td></tr>`;
+  }));
+  var order_obj = order_el.sortable();
+  order_el.disableSelection();
   $("#add_group").dialog({
     title: 'Create Group',
+    width: 'auto',
     modal: true,
     resizable: false,
-    buttons: {
-      Cancel: function() {
-        $(this).dialog("close");
+    buttons: [
+      {
+        text: "Cancel",
+        click: function() {
+          $(this).dialog("close");
+        }
       },
-      "Create Group": function() {
-        var data = $('#add_group > form').serialize();
-        var url = get_cluster_remote_url() + "add_group";
-        ajax_wrapper({
-          type: "POST",
-          url: url,
-          data: data,
-          success: function() {
-            Pcs.update();
-            $("#add_group").dialog("close");
-          },
-          error: function (xhr, status, error) {
-            alert(
-              "Error creating group "
-              + ajax_simple_error(xhr, status, error)
-            );
-            $("#add_group").dialog("close");
-          }
-        });
+      {
+        text: "Create Group",
+        id: "add_group_submit_btn",
+        click: function() {
+          var dialog_obj = $(this);
+          var submit_btn_obj = dialog_obj.parent().find(
+            "#add_group_submit_btn"
+          );
+          submit_btn_obj.button("option", "disabled", true);
+
+          ajax_wrapper({
+            type: "POST",
+            url: get_cluster_remote_url() + "add_group",
+            data: {
+              resource_group: $(
+                '#add_group:visible input[name=resource_group]'
+              ).val(),
+              resources: order_obj.sortable(
+                "toArray", {attribute: "value"}
+              ).join(" ")
+            },
+            success: function() {
+              submit_btn_obj.button("option", "disabled", false);
+              Pcs.update();
+              dialog_obj.dialog("close");
+            },
+            error: function (xhr, status, error) {
+              alert(
+                "Error creating group "
+                + ajax_simple_error(xhr, status, error)
+              );
+              submit_btn_obj.button("option", "disabled", false);
+            }
+          });
+        }
       }
-    }
+    ]
   });
 }
 
@@ -1134,8 +1161,8 @@ function hover_out(o) {
 }
 
 function reload_current_resource() {
-  tree_view_onclick(curResource(), true);
-  tree_view_onclick(curStonith(), true);
+  tree_view_onclick(curResource());
+  tree_view_onclick(curStonith());
 }
 
 function load_row(node_row, ac, cur_elem, containing_elem, also_set, initial_load){
@@ -1262,7 +1289,9 @@ function remove_nodes(ids, force) {
 }
 
 function remove_resource(ids, force) {
-  var data = {};
+  var data = {
+    no_error_if_not_exists: true
+  };
   if (force) {
     data["force"] = force;
   }
@@ -1287,12 +1316,27 @@ function remove_resource(ids, force) {
       Pcs.update();
     },
     error: function (xhr, status, error) {
-      error = $.trim(error)
-      var message = "Unable to remove resources (" + error + ")";
+      error = $.trim(error);
+      var message = "";
       if (
-        (xhr.responseText.substring(0,6) == "Error:") || ("Forbidden" == error)
+        status == "timeout" ||
+        error == "timeout" ||
+        xhr.responseText == '{"noresponse":true}'
       ) {
-        message += "\n\n" + xhr.responseText.replace("--force", "'Enforce removal'");
+        message = "Operation takes longer to complete than expected.";
+      } else {
+        message = "Unable to remove resources (" + error + ")";
+        if (
+          (xhr.responseText.substring(0, 6) == "Error:") ||
+          ("Forbidden" == error)
+        ) {
+          message += "\n\n" + xhr.responseText.replace(
+            "--force", "'Enforce removal'"
+          );
+          alert(message);
+          $("#verify_remove_submit_btn").button("option", "disabled", false);
+          return;
+        }
       }
       alert(message);
       $("#dialog_verify_remove_resources.ui-dialog-content").each(
@@ -1764,8 +1808,10 @@ function remove_acl_item(id,item) {
       data["acl_perm_id"] = id.attr("acl_perm_id");
       item_label = "permission"
       break;
-    case "usergroup":
+    case "group":
+    case "user":
       data["item"] = "usergroup";
+      data["item_type"] = item;
       data["usergroup_id"] = id.attr("usergroup_id")
       data["role_id"] = id.attr("role_id")
       item_label = "user / group"
@@ -1916,6 +1962,7 @@ function get_status_value(status) {
     maintenance: 2,
     "partially running": 2,
     disabled: 3,
+    unmanaged: 3,
     unknown: 4,
     ok: 5,
     running: 5,
@@ -1932,7 +1979,8 @@ function status_comparator(a,b) {
   return valA - valB;
 }
 
-function get_status_icon_class(status_val) {
+function get_status_icon_class(status_val, is_unmanaged) {
+  var is_unmanaged = typeof is_unmanaged !== 'undefined' ? is_unmanaged : false;
   switch (status_val) {
     case get_status_value("error"):
       return "error";
@@ -1940,15 +1988,16 @@ function get_status_icon_class(status_val) {
     case get_status_value("warning"):
       return "warning";
     case get_status_value("ok"):
-      return "check";
+      return is_unmanaged ? "warning" : "check";
     default:
       return "x";
   }
 }
 
-function get_status_color(status_val) {
+function get_status_color(status_val, is_unmanaged) {
+  var is_unmanaged = typeof is_unmanaged !== 'undefined' ? is_unmanaged : false;
   if (status_val == get_status_value("ok")) {
-    return "green";
+    return is_unmanaged? "orange" : "green";
   }
   else if (status_val == get_status_value("warning") || status_val == get_status_value("unknown") || status_val == get_status_value('disabled')) {
     return "orange";
@@ -2032,8 +2081,13 @@ function fix_auth_of_cluster() {
   });
 }
 
-function get_tree_view_element_id(element) {
-  return $(element).parents('table.tree-element')[0].id;
+function get_tree_view_resource_id(element) {
+  var suffix = '-treeview-element';
+  var element_id = $(element).parents('table.tree-element')[0].id;
+  if (element_id && element_id.endsWith(suffix)) {
+    return element_id.substr(0, element_id.lastIndexOf(suffix));
+  }
+  return null;
 }
 
 function get_list_view_element_id(element) {
@@ -2046,6 +2100,7 @@ function auto_show_hide_constraints() {
     "ordering_constraints",
     "ordering_set_constraints",
     "colocation_constraints",
+    "colocation_set_constraints",
     "ticket_constraints",
     "ticket_set_constraints",
     "meta_attributes",
@@ -2097,34 +2152,30 @@ function update_instance_attributes(resource_id) {
   }, res_obj.get("stonith"));
 }
 
-function tree_view_onclick(resource_id, auto) {
-  auto = typeof auto !== 'undefined' ? auto : false;
+function tree_view_onclick(resource_id) {
   var resource_obj = Pcs.resourcesContainer.get_resource_by_id(resource_id);
   if (!resource_obj) {
     console.log("Resource " + resource_id + "not found.");
     return;
   }
   if (resource_obj.get('stonith')) {
-    Pcs.resourcesContainer.set('cur_fence', resource_obj);
-    if (!auto) {
+    if (window.location.hash.startsWith("#/fencedevices")) {
       window.location.hash = "/fencedevices/" + resource_id;
-      update_instance_attributes(resource_id);
     }
+    Pcs.resourcesContainer.set('cur_fence', resource_obj);
   } else {
-    Pcs.resourcesContainer.set('cur_resource', resource_obj);
-
-    if (!auto) {
+    if (window.location.hash.startsWith("#/resources")) {
       window.location.hash = "/resources/" + resource_id;
-      update_instance_attributes(resource_id);
     }
+    Pcs.resourcesContainer.set('cur_resource', resource_obj);
     auto_show_hide_constraints();
   }
-
+  update_instance_attributes(resource_id);
   tree_view_select(resource_id);
 }
 
 function tree_view_select(element_id) {
-  var e = $('#' + element_id);
+  var e = $(`#${element_id}-treeview-element`);
   var view = e.parents('table.tree-view');
   view.find('div.arrow').hide();
   view.find('tr.children').hide();
@@ -2136,15 +2187,6 @@ function tree_view_select(element_id) {
   e.find('tr.children').show();
 }
 
-function list_view_select(element_id) {
-  var e = $('#' + element_id);
-  var view = e.parents('table.list-view');
-  view.find('div.arrow').hide();
-  view.find('tr.list-view-element').removeClass("node_selected");
-  e.addClass('node_selected');
-  e.find('div.arrow').show();
-}
-
 function tree_view_checkbox_onchange(element) {
   var e = $(element);
   var children = $(element).closest(".tree-element").find(".children" +
@@ -2246,24 +2288,24 @@ function resource_ungroup(group_id) {
   });
 }
 
-function resource_change_group(resource_id, group_id) {
+function resource_change_group(resource_id, form) {
   if (resource_id == null) {
     return;
   }
   show_loading_screen();
   var resource_obj = Pcs.resourcesContainer.get_resource_by_id(resource_id);
   var data = {
-    resource_id: resource_id,
-    group_id: group_id
+    resource_id: resource_id
   };
+  $.each($(form).serializeArray(), function(_, item) {
+    data[item.name] = item.value;
+  });
 
-  if (resource_obj.get('parent')) {
-    if (resource_obj.get('parent').get('id') == group_id) {
-      return;
-    }
-    if (resource_obj.get('parent').get('class_type') == 'group') {
-      data['old_group_id'] = resource_obj.get('parent').get('id');
-    }
+  if (
+    resource_obj.get('parent') &&
+    resource_obj.get('parent').get('class_type') == 'group'
+  ) {
+    data['old_group_id'] = resource_obj.get('parent').get('id');
   }
 
   ajax_wrapper({
@@ -2949,3 +2991,64 @@ function sbd_status_dialog() {
     buttons: buttonsOpts
   });
 }
+
+function unmanage_resource(resource_id) {
+  if (!resource_id) {
+    return;
+  }
+  fade_in_out("#resource_unmanage_link");
+  ajax_wrapper({
+    type: 'POST',
+    url: get_cluster_remote_url() + "unmanage_resource",
+    data: {
+      resource_list_json: JSON.stringify([resource_id]),
+    },
+    timeout: pcs_timeout,
+    complete: function() {
+      Pcs.update();
+    },
+    error: function (xhr, status, error) {
+      alert(
+        `Unable to unmanage '${resource_id}': ` +
+        ajax_simple_error(xhr, status, error)
+      );
+    },
+  });
+}
+
+function manage_resource(resource_id) {
+  if (!resource_id) {
+    return;
+  }
+  fade_in_out("#resource_manage_link");
+  ajax_wrapper({
+    type: 'POST',
+    url: get_cluster_remote_url() + "manage_resource",
+    data: {
+      resource_list_json: JSON.stringify([resource_id]),
+    },
+    timeout: pcs_timeout,
+    complete: function() {
+      Pcs.update();
+    },
+    error: function (xhr, status, error) {
+      alert(
+        `Unable to manage '${resource_id}': ` +
+        ajax_simple_error(xhr, status, error)
+      );
+    }
+  });
+}
+
+function show_add_resource_dialog() {
+  var new_resource_group_selector_id = $(
+    "#new_resource_agent .group-selector"
+  ).attr("id");
+  Ember.View.views[new_resource_group_selector_id].set(
+    "group_select_value", null
+  );
+  $('#new_resource_agent').dialog({
+    title: 'Add Resource',
+    modal:true, width: 'auto'
+  });
+}
