@@ -83,6 +83,10 @@ def remote(params, request, auth_user)
       :qdevice_client_disable => method(:qdevice_client_disable),
       :qdevice_client_start => method(:qdevice_client_start),
       :qdevice_client_stop => method(:qdevice_client_stop),
+      :booth_set_config => method(:booth_set_config),
+      :booth_save_files => method(:booth_save_files),
+      :booth_get_config => method(:booth_get_config),
+
   }
   remote_cmd_with_pacemaker = {
       :pacemaker_node_status => method(:remote_pacemaker_node_status),
@@ -116,7 +120,14 @@ def remote(params, request, auth_user)
       :set_resource_utilization => method(:set_resource_utilization),
       :set_node_utilization => method(:set_node_utilization),
       :get_resource_agent_metadata => method(:get_resource_agent_metadata),
-      :get_fence_agent_metadata => method(:get_fence_agent_metadata)
+      :get_fence_agent_metadata => method(:get_fence_agent_metadata),
+      :manage_resource => method(:manage_resource),
+      :unmanage_resource => method(:unmanage_resource),
+      :create_alert => method(:create_alert),
+      :update_alert => method(:update_alert),
+      :create_recipient => method(:create_recipient),
+      :update_recipient => method(:update_recipient),
+      :remove_alerts_and_recipients => method("remove_alerts_and_recipients"),
   }
 
   command = params[:command].to_sym
@@ -328,9 +339,8 @@ end
 def node_standby(params, request, auth_user)
   if params[:name]
     code, response = send_request_with_token(
-      auth_user, params[:name], 'node_standby', true, {"node"=>params[:name]}
+      auth_user, params[:name], 'node_standby', true
     )
-    # data={"node"=>params[:name]} for backward compatibility with older versions of pcs/pcsd
   else
     if not allowed_for_local_cluster(auth_user, Permissions::WRITE)
       return 403, 'Permission denied'
@@ -344,9 +354,8 @@ end
 def node_unstandby(params, request, auth_user)
   if params[:name]
     code, response = send_request_with_token(
-      auth_user, params[:name], 'node_unstandby', true, {"node"=>params[:name]}
+      auth_user, params[:name], 'node_unstandby', true
     )
-    # data={"node"=>params[:name]} for backward compatibility with older versions of pcs/pcsd
   else
     if not allowed_for_local_cluster(auth_user, Permissions::WRITE)
       return 403, 'Permission denied'
@@ -769,9 +778,19 @@ def get_sw_versions(params, request, auth_user)
 end
 
 def remote_node_available(params, request, auth_user)
-  if (not ISRHEL6 and File.exist?(Cfgsync::CorosyncConf.file_path)) or (ISRHEL6 and File.exist?(Cfgsync::ClusterConf.file_path)) or File.exist?("/var/lib/pacemaker/cib/cib.xml")
+  if (
+    (not ISRHEL6 and File.exist?(Cfgsync::CorosyncConf.file_path)) or
+    (ISRHEL6 and File.exist?(Cfgsync::ClusterConf.file_path)) or
+    File.exist?("/var/lib/pacemaker/cib/cib.xml")
+  )
     return JSON.generate({:node_available => false})
   end
+  if pacemaker_remote_running?()
+    return JSON.generate({
+      :node_available => false,
+      :pacemaker_remote => true,
+    })
+  end
   return JSON.generate({:node_available => true})
 end
 
@@ -827,8 +846,15 @@ def remote_remove_nodes(params, request, auth_user)
   stdout, stderr, retval = run_cmd(
     auth_user, PCS, "cluster", "stop", *stop_params
   )
-  if retval != 0
-    return [400, stderr.join]
+  if retval != 0 and not params['force']
+    # If forced, keep going even if unable to stop all nodes (they may be dead).
+    # Add info this error is forceable if pcs did not do it (e.g. when unable
+    # to connect to some nodes).
+    message = stderr.join
+    if not message.include?(', use --force to override')
+      message += ', use --force to override'
+    end
+    return [400, message]
   end
 
   node_list.each {|node|
@@ -1038,6 +1064,8 @@ def node_status(params, request, auth_user)
     :cman => node.cman,
     :corosync_enabled => node.corosync_enabled,
     :pacemaker_enabled => node.pacemaker_enabled,
+    :pacemaker_remote => node.services[:pacemaker_remote][:running],
+    :pacemaker_remote_enabled => node.services[:pacemaker_remote][:enabled],
     :pcsd_enabled => node.pcsd_enabled,
     :corosync_online => status[:corosync_online],
     :corosync_offline => status[:corosync_offline],
@@ -1396,21 +1424,23 @@ def update_resource (params, request, auth_user)
 
   param_line = getParamList(params)
   if not params[:resource_id]
-    out, stderr, retval = run_cmd(
-      auth_user,
-      PCS, "resource", "create", params[:name], params[:resource_type],
-      *param_line
-    )
+    cmd = [PCS, "resource", "create", params[:name], params[:resource_type]]
+    cmd += param_line
+    if params[:resource_group] and params[:resource_group] != ""
+      cmd += ['--group', params[:resource_group]]
+      if (
+        ['before', 'after'].include?(params[:in_group_position]) and
+        params[:in_group_reference_resource_id]
+      )
+        cmd << "--#{params[:in_group_position]}"
+        cmd << params[:in_group_reference_resource_id]
+      end
+      resource_group = params[:resource_group]
+    end
+    out, stderr, retval = run_cmd(auth_user, *cmd)
     if retval != 0
       return JSON.generate({"error" => "true", "stderr" => stderr, "stdout" => out})
     end
-    if params[:resource_group] and params[:resource_group] != ""
-      run_cmd(
-        auth_user,
-        PCS, "resource","group", "add", params[:resource_group], params[:name]
-      )
-      resource_group = params[:resource_group]
-    end
 
     if params[:resource_clone] and params[:resource_clone] != ""
       name = resource_group ? resource_group : params[:name]
@@ -1442,10 +1472,18 @@ def update_resource (params, request, auth_user)
         )
       end
     else
-      run_cmd(
-        auth_user, PCS, "resource", "group", "add", params[:resource_group],
+      cmd = [
+        PCS, "resource", "group", "add", params[:resource_group],
         params[:resource_id]
+      ]
+      if (
+        ['before', 'after'].include?(params[:in_group_position]) and
+        params[:in_group_reference_resource_id]
       )
+        cmd << "--#{params[:in_group_position]}"
+        cmd << params[:in_group_reference_resource_id]
+      end
+      run_cmd(auth_user, *cmd)
     end
   end
 
@@ -1515,7 +1553,7 @@ def get_avail_fence_agents(params, request, auth_user)
   if not allowed_for_local_cluster(auth_user, Permissions::READ)
     return 403, 'Permission denied'
   end
-  agents = getFenceAgents()
+  agents = getFenceAgents(auth_user)
   return JSON.generate(agents)
 end
 
@@ -1546,10 +1584,10 @@ def remove_resource(params, request, auth_user)
       end
       cmd = [PCS, '-f', tmp_file.path, 'resource', 'disable']
       resource_list.each { |resource|
-        _, err, retval = run_cmd(user, *cmd, resource)
+        out, err, retval = run_cmd(user, *(cmd + [resource]))
         if retval != 0
           unless (
-            err.join('').index('unable to find a resource') != -1 and
+            (out + err).join('').include?('unable to find a resource') and
             no_error_if_not_exists
           )
             errors += "Unable to stop resource '#{resource}': #{err.join('')}"
@@ -1584,7 +1622,10 @@ def remove_resource(params, request, auth_user)
     end
     out, err, retval = run_cmd(auth_user, *cmd)
     if retval != 0
-      unless out.index(' does not exist.') != -1 and no_error_if_not_exists
+      unless (
+        (out + err).join('').include?(' does not exist.') and
+        no_error_if_not_exists
+      )
         errors += err.join(' ').strip + "\n"
       end
     end
@@ -1704,7 +1745,7 @@ def remove_acl_remote(params, request, auth_user)
     retval = remove_acl_permission(auth_user, params["acl_perm_id"])
   elsif params["item"] == "usergroup"
     retval = remove_acl_usergroup(
-      auth_user, params["role_id"],params["usergroup_id"]
+      auth_user, params["role_id"],params["usergroup_id"], params["item_type"]
     )
   else
     retval = "Error: Unknown removal request"
@@ -1821,6 +1862,11 @@ def add_constraint_set_remote(params, request, auth_user)
       auth_user,
       params["resources"].values, params["force"], !params['disable_autocorrect']
     )
+  when "col"
+    retval, error = add_colocation_set_constraint(
+      auth_user,
+      params["resources"].values, params["force"], !params['disable_autocorrect']
+    )
   when "ticket"
     unless params["options"]["ticket"]
       return [400, "Error adding constraint ticket: option ticket missing"]
@@ -2074,10 +2120,17 @@ def resource_change_group(params, request, auth_user)
     end
     return 200
   end
-  _, stderr, retval = run_cmd(
-    auth_user,
+  cmd = [
     PCS, 'resource', 'group', 'add', params[:group_id], params[:resource_id]
+  ]
+  if (
+  ['before', 'after'].include?(params[:in_group_position]) and
+    params[:in_group_reference_resource_id]
   )
+    cmd << "--#{params[:in_group_position]}"
+    cmd << params[:in_group_reference_resource_id]
+  end
+  _, stderr, retval = run_cmd(auth_user, *cmd)
   if retval != 0
     return [400, "Unable to add resource '#{params[:resource_id]}' to " +
       "group '#{params[:group_id]}': #{stderr.join('')}"
@@ -2190,7 +2243,7 @@ def set_node_utilization(params, reqest, auth_user)
 
   if retval != 0
     return [400, "Unable to set utilization '#{name}=#{value}' for node " +
-      "'#{res_id}': #{stderr.join('')}"
+      "'#{node}': #{stderr.join('')}"
     ]
   end
   return 200
@@ -2258,9 +2311,9 @@ def check_sbd(param, request, auth_user)
   end
   out = {
     :sbd => {
-      :installed => is_service_installed?('sbd'),
-      :enabled => is_service_enabled?('sbd'),
-      :running => is_service_running?('sbd')
+      :installed => is_service_installed?(get_sbd_service_name()),
+      :enabled => is_service_enabled?(get_sbd_service_name()),
+      :running => is_service_running?(get_sbd_service_name())
     }
   }
   watchdog = param[:watchdog]
@@ -2329,7 +2382,7 @@ def sbd_disable(param, request, auth_user)
   unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
     return 403, 'Permission denied'
   end
-  if disable_service('sbd')
+  if disable_service(get_sbd_service_name())
     msg = 'SBD disabled'
     $logger.info(msg)
     return [200, msg]
@@ -2344,7 +2397,7 @@ def sbd_enable(param, request, auth_user)
   unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
     return 403, 'Permission denied'
   end
-  if enable_service('sbd')
+  if enable_service(get_sbd_service_name())
     msg = 'SBD enabled'
     $logger.info(msg)
     return [200, msg]
@@ -2589,3 +2642,324 @@ def qdevice_client_start(param, request, auth_user)
     return [400, msg]
   end
 end
+
+def manage_resource(param, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+  unless param[:resource_list_json]
+    return [400, "Required parameter 'resource_list_json' is missing."]
+  end
+  begin
+    resource_list = JSON.parse(param[:resource_list_json])
+    _, err, retval = run_cmd(
+      auth_user, PCS, 'resource', 'manage', *resource_list
+    )
+    if retval != 0
+      return [400, err.join('')]
+    end
+    return [200, '']
+  rescue JSON::ParserError
+    return [400, 'Invalid input data format']
+  end
+end
+
+def unmanage_resource(param, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+  unless param[:resource_list_json]
+    return [400, "Required parameter 'resource_list_json' is missing."]
+  end
+  begin
+    resource_list = JSON.parse(param[:resource_list_json])
+    _, err, retval = run_cmd(
+      auth_user, PCS, 'resource', 'unmanage', *resource_list
+    )
+    if retval != 0
+      return [400, err.join('')]
+    end
+    return [200, '']
+  rescue JSON::ParserError
+    return [400, 'Invalid input data format']
+  end
+end
+
+def booth_set_config(params, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+  begin
+    unless params[:data_json]
+      return [400, "Missing required parameter 'data_json'"]
+    end
+    data = JSON.parse(params[:data_json], {:symbolize_names => true})
+  rescue JSON::ParserError
+    return [400, 'Invalid input data format']
+  end
+  config = data[:config]
+  authfile = data[:authfile]
+  return [400, 'Invalid input data format'] unless (
+    config and config[:name] and config[:data]
+  )
+  return [400, 'Invalid input data format'] if (
+    authfile and (not authfile[:name] or not authfile[:data])
+  )
+  begin
+    write_booth_config(config[:name], config[:data])
+    if authfile
+      write_booth_authfile(authfile[:name], authfile[:data])
+    end
+  rescue InvalidFileNameException => e
+    return [400, "Invalid format of config/key file name '#{e.message}'"]
+  rescue => e
+    msg = "Unable to save booth configuration: #{e.message}"
+    $logger.error(msg)
+    return [400, msg]
+  end
+  msg = 'Booth configuration saved.'
+  $logger.info(msg)
+  return [200, msg]
+end
+
+def booth_save_files(params, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+  begin
+    data = JSON.parse(params[:data_json], {:symbolize_names => true})
+    data.each { |file|
+      unless file[:name] and file[:data]
+        return [400, 'Invalid input data format']
+      end
+      if file[:name].include?('/')
+        return [400, "Invalid file name format '#{file[:name]}'"]
+      end
+    }
+  rescue JSON::ParserError, NoMethodError
+    return [400, 'Invalid input data format']
+  end
+  rewrite_existing = (
+  params.include?('rewrite_existing') || params.include?(:rewrite_existing)
+  )
+
+  conflict_files = []
+  data.each { |file|
+    next unless File.file?(File.join(BOOTH_CONFIG_DIR, file[:name]))
+    if file[:is_authfile]
+      cur_data = read_booth_authfile(file[:name])
+    else
+      cur_data = read_booth_config(file[:name])
+    end
+    if cur_data != file[:data]
+      conflict_files << file[:name]
+    end
+  }
+
+  write_failed = {}
+  saved_files = []
+  data.each { |file|
+    next if conflict_files.include?(file[:name]) and not rewrite_existing
+    begin
+      if file[:is_authfile]
+        write_booth_authfile(file[:name], file[:data])
+      else
+        write_booth_config(file[:name], file[:data])
+      end
+      saved_files << file[:name]
+    rescue => e
+      msg = "Unable to save file (#{file[:name]}): #{e.message}"
+      $logger.error(msg)
+      write_failed[file[:name]] = e
+    end
+  }
+  return [200, JSON.generate({
+    :existing => conflict_files,
+    :saved => saved_files,
+    :failed => write_failed
+  })]
+end
+
+def booth_get_config(params, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::READ)
+    return 403, 'Permission denied'
+  end
+  name = params[:name]
+  if name
+    config_file_name = "#{name}.conf"
+  else
+    config_file_name = 'booth.conf'
+  end
+  if config_file_name.include?('/')
+    return [400, 'Invalid name of booth configuration']
+  end
+  begin
+    config_data = read_booth_config(config_file_name)
+    unless config_data
+      return [400, "Config doesn't exist"]
+    end
+    authfile_name = nil
+    authfile_data = nil
+    authfile_path = get_authfile_from_booth_config(config_data)
+    if authfile_path
+      if File.dirname(authfile_path) != BOOTH_CONFIG_DIR
+        return [
+          400, "Authfile of specified config is not in '#{BOOTH_CONFIG_DIR}'"
+        ]
+      end
+      authfile_name = File.basename(authfile_path)
+      authfile_data = read_booth_authfile(authfile_name)
+    end
+    return [200, JSON.generate({
+      :config => {
+        :name => config_file_name,
+        :data => config_data
+      },
+      :authfile => {
+        :name => authfile_name,
+        :data => authfile_data
+      }
+    })]
+  rescue => e
+    return [400, "Unable to read booth config/key file: #{e.message}"]
+  end
+end
+
+def _hash_to_argument_list(hash)
+  result = []
+  if hash.kind_of?(Hash)
+    hash.each {|key, value|
+      value = '' if value.nil?
+      result << "#{key}=#{value}"
+    }
+  end
+  return result
+end
+
+def create_alert(params, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+  path = params[:path]
+  unless path
+    return [400, 'Missing required parameter: path']
+  end
+  alert_id = params[:alert_id]
+  description = params[:description]
+  meta_attr_list = _hash_to_argument_list(params[:meta_attr])
+  instance_attr_list = _hash_to_argument_list(params[:instance_attr])
+  cmd = [PCS, 'alert', 'create', "path=#{path}"]
+  cmd << "id=#{alert_id}" if alert_id and alert_id != ''
+  cmd << "description=#{description}" if description and description != ''
+  cmd += ['options', *instance_attr_list] if instance_attr_list.any?
+  cmd += ['meta', *meta_attr_list] if meta_attr_list.any?
+  output, stderr, retval = run_cmd(auth_user, *cmd)
+  if retval != 0
+    return [400, "Unable to create alert: #{stderr.join("\n")}"]
+  end
+  return [200, 'Alert created']
+end
+
+def update_alert(params, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+  alert_id = params[:alert_id]
+  unless alert_id
+    return [400, 'Missing required parameter: alert_id']
+  end
+  path = params[:path]
+  description = params[:description]
+  meta_attr_list = _hash_to_argument_list(params[:meta_attr])
+  instance_attr_list = _hash_to_argument_list(params[:instance_attr])
+  cmd = [PCS, 'alert', 'update', alert_id]
+  cmd << "path=#{path}" if path
+  cmd << "description=#{description}" if description
+  cmd += ['options', *instance_attr_list] if instance_attr_list.any?
+  cmd += ['meta', *meta_attr_list] if meta_attr_list.any?
+  output, stderr, retval = run_cmd(auth_user, *cmd)
+  if retval != 0
+    return [400, "Unable to update alert: #{stderr.join("\n")}"]
+  end
+  return [200, 'Alert updated']
+end
+
+def remove_alerts_and_recipients(params, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+  alert_list = params[:alert_list]
+  recipient_list = params[:recipient_list]
+  if recipient_list.kind_of?(Array) and recipient_list.any?
+    output, stderr, retval = run_cmd(
+      auth_user, PCS, 'alert', 'recipient', 'remove', *recipient_list
+    )
+    if retval != 0
+      return [400, "Unable to remove recipients: #{stderr.join("\n")}"]
+    end
+  end
+  if alert_list.kind_of?(Array) and alert_list.any?
+    output, stderr, retval = run_cmd(
+      auth_user, PCS, 'alert', 'remove', *alert_list
+    )
+    if retval != 0
+      return [400, "Unable to remove alerts: #{stderr.join("\n")}"]
+    end
+  end
+  return [200, 'All removed']
+end
+
+def create_recipient(params, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+  alert_id = params[:alert_id]
+  if not alert_id or alert_id.strip! == ''
+    return [400, 'Missing required paramter: alert_id']
+  end
+  value = params[:value]
+  if not value or value == ''
+    return [400, 'Missing required paramter: value']
+  end
+  recipient_id = params[:recipient_id]
+  description = params[:description]
+  meta_attr_list = _hash_to_argument_list(params[:meta_attr])
+  instance_attr_list = _hash_to_argument_list(params[:instance_attr])
+  cmd = [PCS, 'alert', 'recipient', 'add', alert_id, "value=#{value}"]
+  cmd << "id=#{recipient_id}" if recipient_id and recipient_id != ''
+  cmd << "description=#{description}" if description and description != ''
+  cmd += ['options', *instance_attr_list] if instance_attr_list.any?
+  cmd += ['meta', *meta_attr_list] if meta_attr_list.any?
+  output, stderr, retval = run_cmd(auth_user, *cmd)
+  if retval != 0
+    return [400, "Unable to create recipient: #{stderr.join("\n")}"]
+  end
+  return [200, 'Recipient created']
+end
+
+def update_recipient(params, request, auth_user)
+  unless allowed_for_local_cluster(auth_user, Permissions::WRITE)
+    return 403, 'Permission denied'
+  end
+  recipient_id = params[:recipient_id]
+  if not recipient_id or recipient_id.strip! == ''
+    return [400, 'Missing required paramter: recipient_id']
+  end
+  value = params[:value]
+  if value and value.strip! == ''
+    return [400, 'Parameter value canot be empty string']
+  end
+  description = params[:description]
+  meta_attr_list = _hash_to_argument_list(params[:meta_attr])
+  instance_attr_list = _hash_to_argument_list(params[:instance_attr])
+  cmd = [PCS, 'alert', 'recipient', 'update', recipient_id]
+  cmd << "value=#{value}" if value
+  cmd << "description=#{description}" if description
+  cmd += ['options', *instance_attr_list] if instance_attr_list.any?
+  cmd += ['meta', *meta_attr_list] if meta_attr_list.any?
+  output, stderr, retval = run_cmd(auth_user, *cmd)
+  if retval != 0
+    return [400, "Unable to update recipient: #{stderr.join("\n")}"]
+  end
+  return [200, 'Recipient updated']
+end
